@@ -1,6 +1,6 @@
 // SPDX-License-Identifier: Apache-2.0
 // ----------------------------------------------------------------------------
-// Copyright 2020-2021 Arm Limited
+// Copyright 2020-2022 Arm Limited
 //
 // Licensed under the Apache License, Version 2.0 (the "License"); you may not
 // use this file except in compliance with the License. You may obtain a copy
@@ -46,7 +46,7 @@
  * Threading
  * =========
  *
- * In pseudocode, the usage for manual user threading looks like this:
+ * In pseudo-code, the usage for manual user threading looks like this:
  *
  *     // Configure the compressor run
  *     astcenc_config my_config;
@@ -82,7 +82,7 @@
  * Images can be any dimension; there is no requirement to be a multiple of the ASTC block size.
  *
  * Data is always passed in as 4 color components, and accessed as an array of 2D image slices. Data
- * within an image slice is always tightly packed without padding. Addresing looks like this:
+ * within an image slice is always tightly packed without padding. Addressing looks like this:
  *
  *     data[z_coord][y_coord * x_dim * 4 + x_coord * 4    ]   // Red
  *     data[z_coord][y_coord * x_dim * 4 + x_coord * 4 + 1]   // Green
@@ -112,8 +112,8 @@
  * Input images must contain unit-length normalized and should be passed in using a two component
  * swizzle. The astcenc command line tool defaults to an RRRG swizzle, but some developers prefer
  * to use GGGR for compatability with BC5n which will work just as well. The Z component can be
- * recovered programatically in shader code, using knowledge that the vector is unit length and that
- * Z must be positive for a tangent-space normal map.
+ * recovered programmatically in shader code, using knowledge that the vector is unit length and
+ * that Z must be positive for a tangent-space normal map.
  *
  * Decompress-only usage
  * =====================
@@ -142,15 +142,18 @@
  *
  * A normal context is capable of decompressing any ASTC texture, including those generated by other
  * compressors with unknown heuristics. This is the most flexible implementation, but forces the
- * main data tables used by the codec to include entries that are not needed during compressor. This
- * can slow down compression by ~15%. To optimize this use case the context can be created with the
- * ASTCENC_FLG_SELF_DECOMPRESS_ONLY flag. This tells the compressor that it will only be asked to
- * decompress images that it compressed, allowing the size of the context structures to be
- * substantially reduced with a corresponding boost in performance.
+ * data tables used by the codec to include entries that are not needed during compression. This
+ * can slow down context creation by a significant amount, especially for the faster compression
+ * modes where few data table entries are actually used. To optimize this use case the context can
+ * be created with the ASTCENC_FLG_SELF_DECOMPRESS_ONLY flag. This tells the compressor that it will
+ * only be asked to decompress images that it compressed itself, allowing the data tables to
+ * exclude entries that are not needed by the current compression configuration. This reduces the
+ * size of the context data tables in memory and improves context creation performance. Note that,
+ * as of the 3.6 release, this flag no longer affects compression performance.
  *
- * Attempting to decompress an valid image which was created by another compressor, or even another
- * astcenc compressor configuration, may result in blocks returning as solid magenta or NaN values
- * if they use unsupported encodings for that configuration.
+ * Using this flag while attempting to decompress an valid image which was created by another
+ * compressor, or even another astcenc compressor version or configuration, may result in blocks
+ * returning as solid magenta or NaN value error blocks.
  */
 
 #ifndef ASTCENC_INCLUDED
@@ -235,7 +238,7 @@ static const float ASTCENC_PRE_FAST = 10.0f;
 /** @brief The medium quality search preset. */
 static const float ASTCENC_PRE_MEDIUM = 60.0f;
 
-/** @brief The throrough quality search preset. */
+/** @brief The thorough quality search preset. */
 static const float ASTCENC_PRE_THOROUGH = 98.0f;
 
 /** @brief The exhaustive, highest quality, search preset. */
@@ -308,29 +311,6 @@ static const unsigned int ASTCENC_FLG_MAP_NORMAL          = 1 << 0;
  */
 static const unsigned int ASTCENC_FLG_MAP_MASK             = 1 << 1;
 
-/**
- * @brief Enable RGBM map compression.
- *
- * Input data will be treated as HDR data that has been stored in an LDR RGBM-encoded wrapper
- * format. Data must be preprocessed by the user to be in LDR RGBM format before calling the
- * compression function, this flag is only used to control the use of RGBM-specific heuristics and
- * error metrics.
- *
- * IMPORTANT: The ASTC format is prone to bad failure modes with unconstrained RGBM data; very small
- * M values can round to zero due to quantization and result in black or white pixels. It is highly
- * recommended that the minimum value of M used in the encoding is kept above a lower threshold (try
- * 16 or 32). Applying this threshold reduces the number of very dark colors that can be
- * represented, but is still higher precision than 8-bit LDR.
- *
- * When this flag is set the value of @c rgbm_m_scale in the context must be set to the RGBM scale
- * factor used during reconstruction. This defaults to 5 when in RGBM mode.
- *
- * It is recommended that the value of @c cw_a_weight is set to twice the value of the multiplier
- * scale, ensuring that the M value is accurately encoded. This defaults to 10 when in RGBM mode,
- * matching the default scale factor.
- */
-static const unsigned int ASTCENC_FLG_MAP_RGBM             = 1 << 6;
-
 /**
  * @brief Enable alpha weighting.
  *
@@ -366,6 +346,29 @@ static const unsigned int ASTCENC_FLG_DECOMPRESS_ONLY      = 1 << 4;
  */
 static const unsigned int ASTCENC_FLG_SELF_DECOMPRESS_ONLY = 1 << 5;
 
+/**
+ * @brief Enable RGBM map compression.
+ *
+ * Input data will be treated as HDR data that has been stored in an LDR RGBM-encoded wrapper
+ * format. Data must be preprocessed by the user to be in LDR RGBM format before calling the
+ * compression function, this flag is only used to control the use of RGBM-specific heuristics and
+ * error metrics.
+ *
+ * IMPORTANT: The ASTC format is prone to bad failure modes with unconstrained RGBM data; very small
+ * M values can round to zero due to quantization and result in black or white pixels. It is highly
+ * recommended that the minimum value of M used in the encoding is kept above a lower threshold (try
+ * 16 or 32). Applying this threshold reduces the number of very dark colors that can be
+ * represented, but is still higher precision than 8-bit LDR.
+ *
+ * When this flag is set the value of @c rgbm_m_scale in the context must be set to the RGBM scale
+ * factor used during reconstruction. This defaults to 5 when in RGBM mode.
+ *
+ * It is recommended that the value of @c cw_a_weight is set to twice the value of the multiplier
+ * scale, ensuring that the M value is accurately encoded. This defaults to 10 when in RGBM mode,
+ * matching the default scale factor.
+ */
+static const unsigned int ASTCENC_FLG_MAP_RGBM             = 1 << 6;
+
 /**
  * @brief The bit mask of all valid flags.
  */
@@ -405,36 +408,6 @@ struct astcenc_config
 	/** @brief The ASTC block size Z dimension. */
 	unsigned int block_z;
 
-	/** @brief The size of the texel kernel for error weighting (-v). */
-	unsigned int v_rgba_radius;
-
-	/** @brief The mean and stdev component mix for error weighting (-v). */
-	float v_rgba_mean_stdev_mix;
-
-	/** @brief The texel RGB power for error weighting (-v). */
-	float v_rgb_power;
-
-	/** @brief The texel RGB base weight for error weighting (-v). */
-	float v_rgb_base;
-
-	/** @brief The texel RGB mean weight for error weighting (-v). */
-	float v_rgb_mean;
-
-	/** @brief The texel RGB stdev for error weighting (-v). */
-	float v_rgb_stdev;
-
-	/** @brief The texel A power for error weighting (-va). */
-	float v_a_power;
-
-	/** @brief The texel A base weight for error weighting (-va). */
-	float v_a_base;
-
-	/** @brief The texel A mean weight for error weighting (-va). */
-	float v_a_mean;
-
-	/** @brief The texel A stdev for error weighting (-va). */
-	float v_a_stdev;
-
 	/** @brief The red component weight scale for error weighting (-cw). */
 	float cw_r_weight;
 
@@ -452,17 +425,10 @@ struct astcenc_config
 	 *
 	 * It is recommended that this is set to 1 when using FLG_USE_ALPHA_WEIGHT on a texture that
 	 * will be sampled using linear texture filtering to minimize color bleed out of transparent
-	 * texels that are adjcent to non-transparent texels.
+	 * texels that are adjacent to non-transparent texels.
 	 */
 	unsigned int a_scale_radius;
 
-	/**
-	 * @brief The additional weight for block edge texels (-b).
-	 *
-	 * This is generic tool for reducing artefacts visible on block changes.
-	 */
-	float b_deblock_weight;
-
 	/** @brief The RGBM scale factor for the shared multiplier (-rgbm). */
 	float rgbm_m_scale;
 
@@ -513,8 +479,8 @@ struct astcenc_config
 	 * @brief The amount of overshoot needed to early-out mode 0 fast path.
 	 *
 	 * We have a fast-path for mode 0 (1 partition, 1 plane) which uses only essential block modes
-	 * as an initital search. This can short-cut compression for simple blocks, but to avoid
-	 * shortcutting too much we* force this to overshoot the MSE threshold needed to hit the
+	 * as an initial search. This can short-cut compression for simple blocks, but to avoid
+	 * short-cutting too much we force this to overshoot the MSE threshold needed to hit the
 	 * block-local db_limit e.g. 1.0 = no overshoot, 2.0 = need half the error to trigger.
 	 */
 	float tune_mode0_mse_overshoot;
@@ -531,14 +497,14 @@ struct astcenc_config
 	float tune_refinement_mse_overshoot;
 
 	/**
-	 * @brief The threshold for skipping 2.2/3.1/3.2/4.1 trials (-2partitionlimitfactor).
+	 * @brief The threshold for skipping 3.1/4.1 trials (-2partitionlimitfactor).
 	 *
 	 * This option is further scaled for normal maps, so it skips less often.
 	 */
 	float tune_2_partition_early_out_limit_factor;
 
 	/**
-	 * @brief The threshold for skipping 3.2/4.1 trials (-3partitionlimitfactor).
+	 * @brief The threshold for skipping 4.1 trials (-3partitionlimitfactor).
 	 *
 	 * This option is further scaled for normal maps, so it skips less often.
 	 */
@@ -810,7 +776,7 @@ ASTCENC_PUBLIC void astcenc_context_free(
  * advanced content packaging pipelines.
  *
  * @param context   Codec context.
- * @param data      One block of compressesd ASTC data.
+ * @param data      One block of compressed ASTC data.
  * @param info      The output info structure to populate.
  *
  * @return @c ASTCENC_SUCCESS if the block was decoded, or an error otherwise. Note that this
