@@ -47,15 +47,48 @@
  * @param value         The value to convert. This may be outside of the 0-255 range and will be
  *                      clamped before the value is looked up.
  *
- * @return The encoded quantized value. These are not necessarily in the order; the compressor
+ * @return The encoded quantized value. These are not necessarily in order; the compressor
  *         scrambles the values slightly to make hardware implementation easier.
  */
-static inline int quant_color(
+static inline int quant_color_clamp(
 	quant_method quant_level,
 	int value
 ) {
 	value = astc::clamp(value, 0, 255);
-	return color_quant_tables[quant_level][value];
+	return color_quant_tables[quant_level - QUANT_6][value];
+}
+
+/**
+ * @brief Determine the quantized value given a quantization level.
+ *
+ * @param quant_level   The quantization level to use.
+ * @param value         The value to convert. This may be outside of the 0-255 range and will be
+ *                      clamped before the value is looked up.
+ *
+ * @return The encoded quantized value. These are not necessarily in order; the compressor
+ *         scrambles the values slightly to make hardware implementation easier.
+ */
+static inline uint8_t quant_color(
+	quant_method quant_level,
+	int value
+) {
+	return color_quant_tables[quant_level - QUANT_6][value];
+}
+
+/**
+ * @brief Determine the unquantized value given a quantization level.
+ *
+ * @param quant_level   The quantization level to use.
+ * @param value         The value to convert.
+ *
+ * @return The encoded quantized value. These are not necessarily in order; the compressor
+ *         scrambles the values slightly to make hardware implementation easier.
+ */
+static inline uint8_t unquant_color(
+	quant_method quant_level,
+	int value
+) {
+	return color_unquant_tables[quant_level - QUANT_6][value];
 }
 
 /**
@@ -90,26 +123,24 @@ static void quantize_rgb(
 	int ri0b, gi0b, bi0b, ri1b, gi1b, bi1b;
 	float rgb0_addon = 0.5f;
 	float rgb1_addon = 0.5f;
-	int iters = 0;
 	do
 	{
-		ri0 = quant_color(quant_level, astc::flt2int_rd(r0 + rgb0_addon));
-		gi0 = quant_color(quant_level, astc::flt2int_rd(g0 + rgb0_addon));
-		bi0 = quant_color(quant_level, astc::flt2int_rd(b0 + rgb0_addon));
-		ri1 = quant_color(quant_level, astc::flt2int_rd(r1 + rgb1_addon));
-		gi1 = quant_color(quant_level, astc::flt2int_rd(g1 + rgb1_addon));
-		bi1 = quant_color(quant_level, astc::flt2int_rd(b1 + rgb1_addon));
+		ri0 = quant_color_clamp(quant_level, astc::flt2int_rd(r0 + rgb0_addon));
+		gi0 = quant_color_clamp(quant_level, astc::flt2int_rd(g0 + rgb0_addon));
+		bi0 = quant_color_clamp(quant_level, astc::flt2int_rd(b0 + rgb0_addon));
+		ri1 = quant_color_clamp(quant_level, astc::flt2int_rd(r1 + rgb1_addon));
+		gi1 = quant_color_clamp(quant_level, astc::flt2int_rd(g1 + rgb1_addon));
+		bi1 = quant_color_clamp(quant_level, astc::flt2int_rd(b1 + rgb1_addon));
 
-		ri0b = color_unquant_tables[quant_level][ri0];
-		gi0b = color_unquant_tables[quant_level][gi0];
-		bi0b = color_unquant_tables[quant_level][bi0];
-		ri1b = color_unquant_tables[quant_level][ri1];
-		gi1b = color_unquant_tables[quant_level][gi1];
-		bi1b = color_unquant_tables[quant_level][bi1];
+		ri0b = unquant_color(quant_level, ri0);
+		gi0b = unquant_color(quant_level, gi0);
+		bi0b = unquant_color(quant_level, bi0);
+		ri1b = unquant_color(quant_level, ri1);
+		gi1b = unquant_color(quant_level, gi1);
+		bi1b = unquant_color(quant_level, bi1);
 
 		rgb0_addon -= 0.2f;
 		rgb1_addon += 0.2f;
-		iters++;
 	} while (ri0b + gi0b + bi0b > ri1b + gi1b + bi1b);
 
 	output[0] = static_cast<uint8_t>(ri0);
@@ -144,8 +175,8 @@ static void quantize_rgba(
 	float a0 = astc::clamp255f(color0.lane<3>() * scale);
 	float a1 = astc::clamp255f(color1.lane<3>() * scale);
 
-	output[6] = color_quant_tables[quant_level][astc::flt2int_rtn(a0)];
-	output[7] = color_quant_tables[quant_level][astc::flt2int_rtn(a1)];
+	output[6] = quant_color(quant_level, astc::flt2int_rtn(a0));
+	output[7] = quant_color(quant_level, astc::flt2int_rtn(a1));
 
 	quantize_rgb(color0, color1, output, quant_level);
 }
@@ -191,22 +222,22 @@ static bool try_quantize_rgb_blue_contract(
 	}
 
 	// Quantize the inverse-blue-contracted color
-	int ri0 = color_quant_tables[quant_level][astc::flt2int_rtn(r0)];
-	int gi0 = color_quant_tables[quant_level][astc::flt2int_rtn(g0)];
-	int bi0 = color_quant_tables[quant_level][astc::flt2int_rtn(b0)];
+	int ri0 = quant_color(quant_level, astc::flt2int_rtn(r0));
+	int gi0 = quant_color(quant_level, astc::flt2int_rtn(g0));
+	int bi0 = quant_color(quant_level, astc::flt2int_rtn(b0));
 
-	int ri1 = color_quant_tables[quant_level][astc::flt2int_rtn(r1)];
-	int gi1 = color_quant_tables[quant_level][astc::flt2int_rtn(g1)];
-	int bi1 = color_quant_tables[quant_level][astc::flt2int_rtn(b1)];
+	int ri1 = quant_color(quant_level, astc::flt2int_rtn(r1));
+	int gi1 = quant_color(quant_level, astc::flt2int_rtn(g1));
+	int bi1 = quant_color(quant_level, astc::flt2int_rtn(b1));
 
 	// Then unquantize again
-	int ru0 = color_unquant_tables[quant_level][ri0];
-	int gu0 = color_unquant_tables[quant_level][gi0];
-	int bu0 = color_unquant_tables[quant_level][bi0];
+	int ru0 = unquant_color(quant_level, ri0);
+	int gu0 = unquant_color(quant_level, gi0);
+	int bu0 = unquant_color(quant_level, bi0);
 
-	int ru1 = color_unquant_tables[quant_level][ri1];
-	int gu1 = color_unquant_tables[quant_level][gi1];
-	int bu1 = color_unquant_tables[quant_level][bi1];
+	int ru1 = unquant_color(quant_level, ri1);
+	int gu1 = unquant_color(quant_level, gi1);
+	int bu1 = unquant_color(quant_level, bi1);
 
 	// If color #1 is not larger than color #0 then blue-contraction cannot be used. Note that
 	// blue-contraction and quantization change this order, which is why we must test aftwards.
@@ -248,8 +279,8 @@ static int try_quantize_rgba_blue_contract(
 	float a0 = astc::clamp255f(color0.lane<3>() * scale);
 	float a1 = astc::clamp255f(color1.lane<3>() * scale);
 
-	output[6] = color_quant_tables[quant_level][astc::flt2int_rtn(a1)];
-	output[7] = color_quant_tables[quant_level][astc::flt2int_rtn(a0)];
+	output[6] = quant_color(quant_level, astc::flt2int_rtn(a1));
+	output[7] = quant_color(quant_level, astc::flt2int_rtn(a0));
 
 	return try_quantize_rgb_blue_contract(color0, color1, output, quant_level);
 }
@@ -299,13 +330,14 @@ static bool try_quantize_rgb_delta(
 	int b0b = b0a & 0xFF;
 
 	// Quantize then unquantize in order to get a value that we take differences against
-	int r0be = color_quant_tables[quant_level][r0b];
-	int g0be = color_quant_tables[quant_level][g0b];
-	int b0be = color_quant_tables[quant_level][b0b];
+	int r0be = quant_color(quant_level, r0b);
+	int g0be = quant_color(quant_level, g0b);
+	int b0be = quant_color(quant_level, b0b);
+
+	r0b = unquant_color(quant_level, r0be);
+	g0b = unquant_color(quant_level, g0be);
+	b0b = unquant_color(quant_level, b0be);
 
-	r0b = color_unquant_tables[quant_level][r0be];
-	g0b = color_unquant_tables[quant_level][g0be];
-	b0b = color_unquant_tables[quant_level][b0be];
 	r0b |= r0a & 0x100;
 	g0b |= g0a & 0x100;
 	b0b |= b0a & 0x100;
@@ -341,13 +373,13 @@ static bool try_quantize_rgb_delta(
 
 	// Then quantize and unquantize; if this causes either top two bits to flip, then encoding fails
 	// since we have then corrupted either the top bit of the base or the sign bit of the offset
-	int r1de = color_quant_tables[quant_level][r1d];
-	int g1de = color_quant_tables[quant_level][g1d];
-	int b1de = color_quant_tables[quant_level][b1d];
+	int r1de = quant_color(quant_level, r1d);
+	int g1de = quant_color(quant_level, g1d);
+	int b1de = quant_color(quant_level, b1d);
 
-	int r1du = color_unquant_tables[quant_level][r1de];
-	int g1du = color_unquant_tables[quant_level][g1de];
-	int b1du = color_unquant_tables[quant_level][b1de];
+	int r1du = unquant_color(quant_level, r1de);
+	int g1du = unquant_color(quant_level, g1de);
+	int b1du = unquant_color(quant_level, b1de);
 
 	if (((r1d ^ r1du) | (g1d ^ g1du) | (b1d ^ b1du)) & 0xC0)
 	{
@@ -441,13 +473,14 @@ static bool try_quantize_rgb_delta_blue_contract(
 	int b0b = b0a & 0xFF;
 
 	// Quantize, then unquantize in order to get a value that we take differences against.
-	int r0be = color_quant_tables[quant_level][r0b];
-	int g0be = color_quant_tables[quant_level][g0b];
-	int b0be = color_quant_tables[quant_level][b0b];
+	int r0be = quant_color(quant_level, r0b);
+	int g0be = quant_color(quant_level, g0b);
+	int b0be = quant_color(quant_level, b0b);
+
+	r0b = unquant_color(quant_level, r0be);
+	g0b = unquant_color(quant_level, g0be);
+	b0b = unquant_color(quant_level, b0be);
 
-	r0b = color_unquant_tables[quant_level][r0be];
-	g0b = color_unquant_tables[quant_level][g0be];
-	b0b = color_unquant_tables[quant_level][b0be];
 	r0b |= r0a & 0x100;
 	g0b |= g0a & 0x100;
 	b0b |= b0a & 0x100;
@@ -484,13 +517,13 @@ static bool try_quantize_rgb_delta_blue_contract(
 	// Then quantize and  unquantize; if this causes any of the top two bits to flip,
 	// then encoding fails, since we have then corrupted either the top bit of the base
 	// or the sign bit of the offset.
-	int r1de = color_quant_tables[quant_level][r1d];
-	int g1de = color_quant_tables[quant_level][g1d];
-	int b1de = color_quant_tables[quant_level][b1d];
+	int r1de = quant_color(quant_level, r1d);
+	int g1de = quant_color(quant_level, g1d);
+	int b1de = quant_color(quant_level, b1d);
 
-	int r1du = color_unquant_tables[quant_level][r1de];
-	int g1du = color_unquant_tables[quant_level][g1de];
-	int b1du = color_unquant_tables[quant_level][b1de];
+	int r1du = unquant_color(quant_level, r1de);
+	int g1du = unquant_color(quant_level, g1de);
+	int b1du = unquant_color(quant_level, b1de);
 
 	if (((r1d ^ r1du) | (g1d ^ g1du) | (b1d ^ b1du)) & 0xC0)
 	{
@@ -573,8 +606,8 @@ static bool try_quantize_alpha_delta(
 	int a0a = astc::flt2int_rtn(a0);
 	a0a <<= 1;
 	int a0b = a0a & 0xFF;
-	int a0be = color_quant_tables[quant_level][a0b];
-	a0b = color_unquant_tables[quant_level][a0be];
+	int a0be = quant_color(quant_level, a0b);
+	a0b = unquant_color(quant_level, a0be);
 	a0b |= a0a & 0x100;
 	int a1d = astc::flt2int_rtn(a1);
 	a1d <<= 1;
@@ -588,8 +621,8 @@ static bool try_quantize_alpha_delta(
 	a1d &= 0x7F;
 	a1d |= (a0b & 0x100) >> 1;
 
-	int a1de = color_quant_tables[quant_level][a1d];
-	int a1du = color_unquant_tables[quant_level][a1de];
+	int a1de = quant_color(quant_level, a1d);
+	int a1du = unquant_color(quant_level, a1de);
 	if ((a1d ^ a1du) & 0xC0)
 	{
 		return false;
@@ -650,10 +683,10 @@ static bool try_quantize_luminance_alpha_delta(
 
 	int l0b = l0a & 0xFF;
 	int a0b = a0a & 0xFF;
-	int l0be = color_quant_tables[quant_level][l0b];
-	int a0be = color_quant_tables[quant_level][a0b];
-	l0b = color_unquant_tables[quant_level][l0be];
-	a0b = color_unquant_tables[quant_level][a0be];
+	int l0be = quant_color(quant_level, l0b);
+	int a0be = quant_color(quant_level, a0b);
+	l0b = unquant_color(quant_level, l0be);
+	a0b = unquant_color(quant_level, a0be);
 	l0b |= l0a & 0x100;
 	a0b |= a0a & 0x100;
 
@@ -679,10 +712,10 @@ static bool try_quantize_luminance_alpha_delta(
 	l1d |= (l0b & 0x100) >> 1;
 	a1d |= (a0b & 0x100) >> 1;
 
-	int l1de = color_quant_tables[quant_level][l1d];
-	int a1de = color_quant_tables[quant_level][a1d];
-	int l1du = color_unquant_tables[quant_level][l1de];
-	int a1du = color_unquant_tables[quant_level][a1de];
+	int l1de = quant_color(quant_level, l1d);
+	int a1de = quant_color(quant_level, a1d);
+	int l1du = unquant_color(quant_level, l1de);
+	int a1du = unquant_color(quant_level, a1de);
 
 	if ((l1d ^ l1du) & 0xC0)
 	{
@@ -800,16 +833,16 @@ static void quantize_rgbs(
 	float g = astc::clamp255f(color.lane<1>() * scale);
 	float b = astc::clamp255f(color.lane<2>() * scale);
 
-	int ri = color_quant_tables[quant_level][astc::flt2int_rtn(r)];
-	int gi = color_quant_tables[quant_level][astc::flt2int_rtn(g)];
-	int bi = color_quant_tables[quant_level][astc::flt2int_rtn(b)];
+	int ri = quant_color(quant_level, astc::flt2int_rtn(r));
+	int gi = quant_color(quant_level, astc::flt2int_rtn(g));
+	int bi = quant_color(quant_level, astc::flt2int_rtn(b));
 
-	int ru = color_unquant_tables[quant_level][ri];
-	int gu = color_unquant_tables[quant_level][gi];
-	int bu = color_unquant_tables[quant_level][bi];
+	int ru = unquant_color(quant_level, ri);
+	int gu = unquant_color(quant_level, gi);
+	int bu = unquant_color(quant_level, bi);
 
 	float oldcolorsum = hadd_rgb_s(color) * scale;
-	float newcolorsum = (float)(ru + gu + bu);
+	float newcolorsum = static_cast<float>(ru + gu + bu);
 
 	float scalea = astc::clamp1f(color.lane<3>() * (oldcolorsum + 1e-10f) / (newcolorsum + 1e-10f));
 	int scale_idx = astc::flt2int_rtn(scalea * 256.0f);
@@ -818,7 +851,7 @@ static void quantize_rgbs(
 	output[0] = static_cast<uint8_t>(ri);
 	output[1] = static_cast<uint8_t>(gi);
 	output[2] = static_cast<uint8_t>(bi);
-	output[3] = color_quant_tables[quant_level][scale_idx];
+	output[3] = quant_color(quant_level, scale_idx);
 }
 
 /**
@@ -840,8 +873,8 @@ static void quantize_rgbs_alpha(
 	float a0 = astc::clamp255f(color0.lane<3>() * scale);
 	float a1 = astc::clamp255f(color1.lane<3>() * scale);
 
-	output[4] = color_quant_tables[quant_level][astc::flt2int_rtn(a0)];
-	output[5] = color_quant_tables[quant_level][astc::flt2int_rtn(a1)];
+	output[4] = quant_color(quant_level, astc::flt2int_rtn(a0));
+	output[5] = quant_color(quant_level, astc::flt2int_rtn(a1));
 
 	quantize_rgbs(color, output, quant_level);
 }
@@ -875,8 +908,8 @@ static void quantize_luminance(
 		lum1 = avg;
 	}
 
-	output[0] = color_quant_tables[quant_level][astc::flt2int_rtn(lum0)];
-	output[1] = color_quant_tables[quant_level][astc::flt2int_rtn(lum1)];
+	output[0] = quant_color(quant_level, astc::flt2int_rtn(lum0));
+	output[1] = quant_color(quant_level, astc::flt2int_rtn(lum1));
 }
 
 /**
@@ -942,10 +975,10 @@ static void quantize_luminance_alpha(
 		}
 	}
 
-	output[0] = color_quant_tables[quant_level][astc::flt2int_rtn(lum0)];
-	output[1] = color_quant_tables[quant_level][astc::flt2int_rtn(lum1)];
-	output[2] = color_quant_tables[quant_level][astc::flt2int_rtn(a0)];
-	output[3] = color_quant_tables[quant_level][astc::flt2int_rtn(a1)];
+	output[0] = quant_color(quant_level, astc::flt2int_rtn(lum0));
+	output[1] = quant_color(quant_level, astc::flt2int_rtn(lum1));
+	output[2] = quant_color(quant_level, astc::flt2int_rtn(a0));
+	output[3] = quant_color(quant_level, astc::flt2int_rtn(a1));
 }
 
 /**
@@ -968,8 +1001,8 @@ static inline void quantize_and_unquantize_retain_top_two_bits(
 
 	do
 	{
-		quantval = color_quant_tables[quant_level][value];
-		uquantval = color_unquant_tables[quant_level][quantval];
+		quantval = quant_color(quant_level, value);
+		uquantval = unquant_color(quant_level, quantval);
 
 		// Perform looping if the top two bits were modified by quant/unquant
 		perform_loop = (value & 0xC0) != (uquantval & 0xC0);
@@ -1012,8 +1045,8 @@ static inline void quantize_and_unquantize_retain_top_four_bits(
 
 	do
 	{
-		quantval = color_quant_tables[quant_level][value];
-		uquantval = color_unquant_tables[quant_level][quantval];
+		quantval = quant_color(quant_level, value);
+		uquantval = unquant_color(quant_level, quantval);
 
 		// Perform looping if the top four bits were modified by quant/unquant
 		perform_loop = (value & 0xF0) != (uquantval & 0xF0);
@@ -1501,8 +1534,8 @@ static void quantize_hdr_rgb(
 		int a_intval = astc::flt2int_rtn(a_base * mode_scale);
 		int a_lowbits = a_intval & 0xFF;
 
-		int a_quantval = color_quant_tables[quant_level][a_lowbits];
-		int a_uquantval = color_unquant_tables[quant_level][a_quantval];
+		int a_quantval = quant_color(quant_level, a_lowbits);
+		int a_uquantval = unquant_color(quant_level, a_quantval);
 		a_intval = (a_intval & ~0xFF) | a_uquantval;
 		float a_fval = static_cast<float>(a_intval) * mode_rscale;
 
@@ -1726,7 +1759,7 @@ static void quantize_hdr_rgb(
 	for (int i = 0; i < 4; i++)
 	{
 		int idx = astc::flt2int_rtn(vals[i] * 1.0f / 256.0f);
-		output[i] = color_quant_tables[quant_level][idx];
+		output[i] = quant_color(quant_level, idx);
 	}
 
 	for (int i = 4; i < 6; i++)
@@ -1759,8 +1792,8 @@ static void quantize_hdr_rgb_ldr_alpha(
 	float a0 = astc::clamp255f(color0.lane<3>() * scale);
 	float a1 = astc::clamp255f(color1.lane<3>() * scale);
 
-	output[6] = color_quant_tables[quant_level][astc::flt2int_rtn(a0)];
-	output[7] = color_quant_tables[quant_level][astc::flt2int_rtn(a1)];
+	output[6] = quant_color(quant_level, astc::flt2int_rtn(a0));
+	output[7] = quant_color(quant_level, astc::flt2int_rtn(a1));
 
 	quantize_hdr_rgb(color0, color1, output, quant_level);
 }
@@ -1832,9 +1865,9 @@ static void quantize_hdr_luminance_large_range(
 		v1 = lower_v1;
 	}
 
-	// OK; encode.
-	output[0] = color_quant_tables[quant_level][v0];
-	output[1] = color_quant_tables[quant_level][v1];
+	// OK; encode
+	output[0] = quant_color(quant_level, v0);
+	output[1] = quant_color(quant_level, v1);
 }
 
 /**
@@ -1885,8 +1918,8 @@ static bool try_quantize_hdr_luminance_small_range(
 	highval = astc::clamp(highval, 0, 2047);
 
 	v0 = lowval & 0x7F;
-	v0e = color_quant_tables[quant_level][v0];
-	v0d = color_unquant_tables[quant_level][v0e];
+	v0e = quant_color(quant_level, v0);
+	v0d = unquant_color(quant_level, v0e);
 
 	if (v0d < 0x80)
 	{
@@ -1895,8 +1928,8 @@ static bool try_quantize_hdr_luminance_small_range(
 		if (diffval >= 0 && diffval <= 15)
 		{
 			v1 = ((lowval >> 3) & 0xF0) | diffval;
-			v1e = color_quant_tables[quant_level][v1];
-			v1d = color_unquant_tables[quant_level][v1e];
+			v1e = quant_color(quant_level, v1);
+			v1d = unquant_color(quant_level, v1e);
 			if ((v1d & 0xF0) == (v1 & 0xF0))
 			{
 				output[0] = static_cast<uint8_t>(v0e);
@@ -1914,8 +1947,8 @@ static bool try_quantize_hdr_luminance_small_range(
 	highval = astc::clamp(highval, 0, 1023);
 
 	v0 = (lowval & 0x7F) | 0x80;
-	v0e = color_quant_tables[quant_level][v0];
-	v0d = color_unquant_tables[quant_level][v0e];
+	v0e = quant_color(quant_level, v0);
+	v0d = unquant_color(quant_level, v0e);
 	if ((v0d & 0x80) == 0)
 	{
 		return false;
@@ -1929,8 +1962,8 @@ static bool try_quantize_hdr_luminance_small_range(
 	}
 
 	v1 = ((lowval >> 2) & 0xE0) | diffval;
-	v1e = color_quant_tables[quant_level][v1];
-	v1d = color_unquant_tables[quant_level][v1e];
+	v1e = quant_color(quant_level, v1);
+	v1d = unquant_color(quant_level, v1e);
 	if ((v1d & 0xE0) != (v1 & 0xE0))
 	{
 		return false;
@@ -1973,8 +2006,8 @@ static void quantize_hdr_alpha(
 		val1 = (ialpha1 + (128 >> i)) >> (8 - i);
 
 		v6 = (val0 & 0x7F) | ((i & 1) << 7);
-		v6e = color_quant_tables[quant_level][v6];
-		v6d = color_unquant_tables[quant_level][v6e];
+		v6e = quant_color(quant_level, v6);
+		v6d = unquant_color(quant_level, v6e);
 
 		if ((v6 ^ v6d) & 0x80)
 		{
@@ -1992,8 +2025,8 @@ static void quantize_hdr_alpha(
 		}
 
 		v7 = ((i & 2) << 6) | ((val0 >> 7) << (6 - i)) | (diffval & mask);
-		v7e = color_quant_tables[quant_level][v7];
-		v7d = color_unquant_tables[quant_level][v7e];
+		v7e = quant_color(quant_level, v7);
+		v7d = unquant_color(quant_level, v7e);
 
 		static const int testbits[3] { 0xE0, 0xF0, 0xF8 };
 
@@ -2013,8 +2046,8 @@ static void quantize_hdr_alpha(
 	v6 = val0 | 0x80;
 	v7 = val1 | 0x80;
 
-	output[0] = color_quant_tables[quant_level][v6];
-	output[1] = color_quant_tables[quant_level][v7];
+	output[0] = quant_color(quant_level, v6);
+	output[1] = quant_color(quant_level, v7);
 
 	return;
 }
@@ -2047,7 +2080,7 @@ uint8_t pack_color_endpoints(
 	uint8_t* output,
 	quant_method quant_level
 ) {
-	assert(quant_level < 21);
+	assert(QUANT_6 <= quant_level && quant_level <= QUANT_256);
 
 	// We do not support negative colors
 	color0 = max(color0, 0.0f);
