@@ -1,6 +1,6 @@
 // SPDX-License-Identifier: Apache-2.0
 // ----------------------------------------------------------------------------
-// Copyright 2019-2021 Arm Limited
+// Copyright 2019-2022 Arm Limited
 //
 // Licensed under the Apache License, Version 2.0 (the "License"); you may not
 // use this file except in compliance with the License. You may obtain a copy
@@ -36,6 +36,9 @@
 
 #include <cstdio>
 
+// Define convenience intrinsics that are missing on older compilers
+#define astcenc_mm256_set_m128i(m, n) _mm256_insertf128_si256(_mm256_castsi128_si256((n)), (m), 1)
+
 // ============================================================================
 // vfloat8 data type
 // ============================================================================
@@ -164,7 +167,7 @@ struct vint8
 	 */
 	ASTCENC_SIMD_INLINE explicit vint8(const int *p)
 	{
-		m = _mm256_loadu_si256((const __m256i*)p);
+		m = _mm256_loadu_si256(reinterpret_cast<const __m256i*>(p));
 	}
 
 	/**
@@ -173,7 +176,7 @@ struct vint8
 	ASTCENC_SIMD_INLINE explicit vint8(const uint8_t *p)
 	{
 		// _mm_loadu_si64 would be nicer syntax, but missing on older GCC
-		m = _mm256_cvtepu8_epi32(_mm_cvtsi64_si128(*(const long long*)p));
+		m = _mm256_cvtepu8_epi32(_mm_cvtsi64_si128(*reinterpret_cast<const long long*>(p)));
 	}
 
 	/**
@@ -242,7 +245,7 @@ struct vint8
 	 */
 	static ASTCENC_SIMD_INLINE vint8 loada(const int* p)
 	{
-		return vint8(_mm256_load_si256((const __m256i*)p));
+		return vint8(_mm256_load_si256(reinterpret_cast<const __m256i*>(p)));
 	}
 
 	/**
@@ -340,9 +343,9 @@ ASTCENC_SIMD_INLINE vmask8 operator~(vmask8 a)
  *
  * bit0 = lane 0
  */
-ASTCENC_SIMD_INLINE unsigned mask(vmask8 a)
+ASTCENC_SIMD_INLINE unsigned int mask(vmask8 a)
 {
-	return _mm256_movemask_ps(a.m);
+	return static_cast<unsigned int>(_mm256_movemask_ps(a.m));
 }
 
 /**
@@ -354,7 +357,7 @@ ASTCENC_SIMD_INLINE bool any(vmask8 a)
 }
 
 /**
- * @brief True if any lanes are enabled, false otherwise.
+ * @brief True if all lanes are enabled, false otherwise.
  */
 ASTCENC_SIMD_INLINE bool all(vmask8 a)
 {
@@ -461,6 +464,14 @@ ASTCENC_SIMD_INLINE vmask8 operator>(vint8 a, vint8 b)
 	return vmask8(_mm256_cmpgt_epi32(a.m, b.m));
 }
 
+/**
+ * @brief Logical shift left.
+ */
+template <int s> ASTCENC_SIMD_INLINE vint8 lsl(vint8 a)
+{
+	return vint8(_mm256_slli_epi32(a.m, s));
+}
+
 /**
  * @brief Arithmetic shift right.
  */
@@ -503,16 +514,13 @@ ASTCENC_SIMD_INLINE vint8 hmin(vint8 a)
 	m = _mm_min_epi32(m, _mm_shuffle_epi32(m, _MM_SHUFFLE(0,0,0,1)));
 	m = _mm_shuffle_epi32(m, _MM_SHUFFLE(0,0,0,0));
 
-	// This is the most logical implementation, but the convenience intrinsic
-	// is missing on older compilers (supported in g++ 9 and clang++ 9).
-	//__m256i r = _mm256_set_m128i(m, m)
-	__m256i r = _mm256_insertf128_si256(_mm256_castsi128_si256(m), m, 1);
+	__m256i r = astcenc_mm256_set_m128i(m, m);
 	vint8 vmin(r);
 	return vmin;
 }
 
 /**
- * @brief Return the horizontal minimum of a vector.
+ * @brief Return the horizontal maximum of a vector.
  */
 ASTCENC_SIMD_INLINE vint8 hmax(vint8 a)
 {
@@ -521,10 +529,7 @@ ASTCENC_SIMD_INLINE vint8 hmax(vint8 a)
 	m = _mm_max_epi32(m, _mm_shuffle_epi32(m, _MM_SHUFFLE(0,0,0,1)));
 	m = _mm_shuffle_epi32(m, _MM_SHUFFLE(0,0,0,0));
 
-	// This is the most logical implementation, but the convenience intrinsic
-	// is missing on older compilers (supported in g++ 9 and clang++ 9).
-	//__m256i r = _mm256_set_m128i(m, m)
-	__m256i r = _mm256_insertf128_si256(_mm256_castsi128_si256(m), m, 1);
+	__m256i r = astcenc_mm256_set_m128i(m, m);
 	vint8 vmax(r);
 	return vmax;
 }
@@ -534,7 +539,7 @@ ASTCENC_SIMD_INLINE vint8 hmax(vint8 a)
  */
 ASTCENC_SIMD_INLINE void storea(vint8 a, int* p)
 {
-	_mm256_store_si256((__m256i*)p, a.m);
+	_mm256_store_si256(reinterpret_cast<__m256i*>(p), a.m);
 }
 
 /**
@@ -542,7 +547,7 @@ ASTCENC_SIMD_INLINE void storea(vint8 a, int* p)
  */
 ASTCENC_SIMD_INLINE void store(vint8 a, int* p)
 {
-	_mm256_storeu_si256((__m256i*)p, a.m);
+	_mm256_storeu_si256(reinterpret_cast<__m256i*>(p), a.m);
 }
 
 /**
@@ -553,7 +558,7 @@ ASTCENC_SIMD_INLINE void store_nbytes(vint8 a, uint8_t* p)
 	// This is the most logical implementation, but the convenience intrinsic
 	// is missing on older compilers (supported in g++ 9 and clang++ 9).
 	// _mm_storeu_si64(ptr, _mm256_extracti128_si256(v.m, 0))
-	_mm_storel_epi64((__m128i*)p, _mm256_extracti128_si256(a.m, 0));
+	_mm_storel_epi64(reinterpret_cast<__m128i*>(p), _mm256_extracti128_si256(a.m, 0));
 }
 
 /**
@@ -578,35 +583,17 @@ ASTCENC_SIMD_INLINE vint8 pack_low_bytes(vint8 v)
 	__m128i a1 = _mm256_extracti128_si256(a, 1);
 	__m128i b = _mm_unpacklo_epi32(a0, a1);
 
-	// This is the most logical implementation, but the convenience intrinsic
-	// is missing on older compilers (supported in g++ 9 and clang++ 9).
-	//__m256i r = _mm256_set_m128i(b, b)
-	__m256i r = _mm256_insertf128_si256(_mm256_castsi128_si256(b), b, 1);
+	__m256i r = astcenc_mm256_set_m128i(b, b);
 	return vint8(r);
 }
 
 /**
- * @brief Return lanes from @c b if MSB of @c cond is set, else @c a.
+ * @brief Return lanes from @c b if @c cond is set, else @c a.
  */
 ASTCENC_SIMD_INLINE vint8 select(vint8 a, vint8 b, vmask8 cond)
 {
-	// Don't use _mm256_blendv_epi8 directly, as it doesn't give the select on
-	// float sign-bit in the mask behavior which is useful. Performance is the
-	// same, these casts are free.
-	__m256 av = _mm256_castsi256_ps(a.m);
-	__m256 bv = _mm256_castsi256_ps(b.m);
-	return vint8(_mm256_castps_si256(_mm256_blendv_ps(av, bv, cond.m)));
-}
-
-/**
- * @brief Debug function to print a vector of ints.
- */
-ASTCENC_SIMD_INLINE void print(vint8 a)
-{
-	alignas(ASTCENC_VECALIGN) int v[8];
-	storea(a, v);
-	printf("v8_i32:\n  %8d %8d %8d %8d %8d %8d %8d %8d\n",
-	       v[0], v[1], v[2], v[3], v[4], v[5], v[6], v[7]);
+	__m256i condi = _mm256_castps_si256(cond.m);
+	return vint8(_mm256_blendv_epi8(a.m, b.m, condi));
 }
 
 // ============================================================================
@@ -746,6 +733,16 @@ ASTCENC_SIMD_INLINE vfloat8 min(vfloat8 a, vfloat8 b)
 	return vfloat8(_mm256_min_ps(a.m, b.m));
 }
 
+/**
+ * @brief Return the min vector of a vector and a scalar.
+ *
+ * If either lane value is NaN, @c b will be returned for that lane.
+ */
+ASTCENC_SIMD_INLINE vfloat8 min(vfloat8 a, float b)
+{
+	return min(a, vfloat8(b));
+}
+
 /**
  * @brief Return the max vector of two vectors.
  *
@@ -756,6 +753,16 @@ ASTCENC_SIMD_INLINE vfloat8 max(vfloat8 a, vfloat8 b)
 	return vfloat8(_mm256_max_ps(a.m, b.m));
 }
 
+/**
+ * @brief Return the max vector of a vector and a scalar.
+ *
+ * If either lane value is NaN, @c b will be returned for that lane.
+ */
+ASTCENC_SIMD_INLINE vfloat8 max(vfloat8 a, float b)
+{
+	return max(a, vfloat8(b));
+}
+
 /**
  * @brief Return the clamped value between min and max.
  *
@@ -820,13 +827,13 @@ ASTCENC_SIMD_INLINE vfloat8 hmin(vfloat8 a)
 {
 	__m128 vlow = _mm256_castps256_ps128(a.m);
 	__m128 vhigh = _mm256_extractf128_ps(a.m, 1);
-	vlow  = _mm_min_ps(vlow, vhigh);
+	vlow = _mm_min_ps(vlow, vhigh);
 
 	// First do an horizontal reduction.
 	__m128 shuf = _mm_shuffle_ps(vlow, vlow, _MM_SHUFFLE(2, 3, 0, 1));
 	__m128 mins = _mm_min_ps(vlow, shuf);
-	shuf        = _mm_movehl_ps(shuf, mins);
-	mins        = _mm_min_ss(mins, shuf);
+	shuf = _mm_movehl_ps(shuf, mins);
+	mins = _mm_min_ss(mins, shuf);
 
 	// This is the most logical implementation, but the convenience intrinsic
 	// is missing on older compilers (supported in g++ 9 and clang++ 9).
@@ -851,13 +858,13 @@ ASTCENC_SIMD_INLINE vfloat8 hmax(vfloat8 a)
 {
 	__m128 vlow = _mm256_castps256_ps128(a.m);
 	__m128 vhigh = _mm256_extractf128_ps(a.m, 1);
-	vhigh  = _mm_max_ps(vlow, vhigh);
+	vhigh = _mm_max_ps(vlow, vhigh);
 
 	// First do an horizontal reduction.
 	__m128 shuf = _mm_shuffle_ps(vhigh, vhigh, _MM_SHUFFLE(2, 3, 0, 1));
 	__m128 maxs = _mm_max_ps(vhigh, shuf);
-	shuf        = _mm_movehl_ps(shuf,maxs);
-	maxs        = _mm_max_ss(maxs, shuf);
+	shuf = _mm_movehl_ps(shuf,maxs);
+	maxs = _mm_max_ss(maxs, shuf);
 
 	// This is the most logical implementation, but the convenience intrinsic
 	// is missing on older compilers (supported in g++ 9 and clang++ 9).
@@ -886,13 +893,28 @@ ASTCENC_SIMD_INLINE float hadd_s(vfloat8 a)
 }
 
 /**
- * @brief Accumulate the full horizontal sum of a vector.
+ * @brief Return lanes from @c b if @c cond is set, else @c a.
  */
-ASTCENC_SIMD_INLINE void haccumulate(float& accum, vfloat8 a)
+ASTCENC_SIMD_INLINE vfloat8 select(vfloat8 a, vfloat8 b, vmask8 cond)
+{
+	return vfloat8(_mm256_blendv_ps(a.m, b.m, cond.m));
+}
+
+/**
+ * @brief Return lanes from @c b if MSB of @c cond is set, else @c a.
+ */
+ASTCENC_SIMD_INLINE vfloat8 select_msb(vfloat8 a, vfloat8 b, vmask8 cond)
+{
+	return vfloat8(_mm256_blendv_ps(a.m, b.m, cond.m));
+}
+
+/**
+ * @brief Accumulate lane-wise sums for a vector, folded 4-wide.
+ *
+ * This is invariant with 4-wide implementations.
+ */
+ASTCENC_SIMD_INLINE void haccumulate(vfloat4& accum, vfloat8 a)
 {
-	// Two sequential 4-wide accumulates gives invariance with 4-wide code.
-	// Note that this approach gives higher error in the sum; adding the two
-	// smaller numbers together first would be more accurate.
 	vfloat4 lo(_mm256_extractf128_ps(a.m, 0));
 	haccumulate(accum, lo);
 
@@ -901,18 +923,35 @@ ASTCENC_SIMD_INLINE void haccumulate(float& accum, vfloat8 a)
 }
 
 /**
- * @brief Accumulate lane-wise sums for a vector, folded 4-wide.
+ * @brief Accumulate lane-wise sums for a vector.
+ *
+ * This is NOT invariant with 4-wide implementations.
  */
-ASTCENC_SIMD_INLINE void haccumulate(vfloat4& accum, vfloat8 a)
+ASTCENC_SIMD_INLINE void haccumulate(vfloat8& accum, vfloat8 a)
 {
-	// Two sequential 4-wide accumulates gives invariance with 4-wide code.
-	// Note that this approach gives higher error in the sum; adding the two
-	// smaller numbers together first would be more accurate.
-	vfloat4 lo(_mm256_extractf128_ps(a.m, 0));
-	haccumulate(accum, lo);
+	accum += a;
+}
 
-	vfloat4 hi(_mm256_extractf128_ps(a.m, 1));
-	haccumulate(accum, hi);
+/**
+ * @brief Accumulate masked lane-wise sums for a vector, folded 4-wide.
+ *
+ * This is invariant with 4-wide implementations.
+ */
+ASTCENC_SIMD_INLINE void haccumulate(vfloat4& accum, vfloat8 a, vmask8 m)
+{
+	a = select(vfloat8::zero(), a, m);
+	haccumulate(accum, a);
+}
+
+/**
+ * @brief Accumulate masked lane-wise sums for a vector.
+ *
+ * This is NOT invariant with 4-wide implementations.
+ */
+ASTCENC_SIMD_INLINE void haccumulate(vfloat8& accum, vfloat8 a, vmask8 m)
+{
+	a = select(vfloat8::zero(), a, m);
+	haccumulate(accum, a);
 }
 
 /**
@@ -923,14 +962,6 @@ ASTCENC_SIMD_INLINE vfloat8 sqrt(vfloat8 a)
 	return vfloat8(_mm256_sqrt_ps(a.m));
 }
 
-/**
- * @brief Return lanes from @c b if MSB of @c cond is set, else @c a.
- */
-ASTCENC_SIMD_INLINE vfloat8 select(vfloat8 a, vfloat8 b, vmask8 cond)
-{
-	return vfloat8(_mm256_blendv_ps(a.m, b.m, cond.m));
-}
-
 /**
  * @brief Load a vector of gathered results from an array;
  */
@@ -963,6 +994,24 @@ ASTCENC_SIMD_INLINE vint8 float_to_int(vfloat8 a)
 	return vint8(_mm256_cvttps_epi32(a.m));
 }
 
+/**
+ * @brief Return a integer value for a float vector, using round-to-nearest.
+ */
+ASTCENC_SIMD_INLINE vint8 float_to_int_rtn(vfloat8 a)
+{
+	a = round(a);
+	return vint8(_mm256_cvttps_epi32(a.m));
+}
+
+
+/**
+ * @brief Return a float value for an integer vector.
+ */
+ASTCENC_SIMD_INLINE vfloat8 int_to_float(vint8 a)
+{
+	return vfloat8(_mm256_cvtepi32_ps(a.m));
+}
+
 /**
  * @brief Return a float value as an integer bit pattern (i.e. no conversion).
  *
@@ -987,6 +1036,148 @@ ASTCENC_SIMD_INLINE vfloat8 int_as_float(vint8 a)
 	return vfloat8(_mm256_castsi256_ps(a.m));
 }
 
+/**
+ * @brief Prepare a vtable lookup table for use with the native SIMD size.
+ */
+ASTCENC_SIMD_INLINE void vtable_prepare(vint4 t0, vint8& t0p)
+{
+	// AVX2 duplicates the table within each 128-bit lane
+	__m128i t0n = t0.m;
+	t0p = vint8(astcenc_mm256_set_m128i(t0n, t0n));
+}
+
+/**
+ * @brief Prepare a vtable lookup table for use with the native SIMD size.
+ */
+ASTCENC_SIMD_INLINE void vtable_prepare(vint4 t0, vint4 t1, vint8& t0p, vint8& t1p)
+{
+	// AVX2 duplicates the table within each 128-bit lane
+	__m128i t0n = t0.m;
+	t0p = vint8(astcenc_mm256_set_m128i(t0n, t0n));
+
+	__m128i t1n = _mm_xor_si128(t0.m, t1.m);
+	t1p = vint8(astcenc_mm256_set_m128i(t1n, t1n));
+}
+
+/**
+ * @brief Prepare a vtable lookup table for use with the native SIMD size.
+ */
+ASTCENC_SIMD_INLINE void vtable_prepare(
+	vint4 t0, vint4 t1, vint4 t2, vint4 t3,
+	vint8& t0p, vint8& t1p, vint8& t2p, vint8& t3p)
+{
+	// AVX2 duplicates the table within each 128-bit lane
+	__m128i t0n = t0.m;
+	t0p = vint8(astcenc_mm256_set_m128i(t0n, t0n));
+
+	__m128i t1n = _mm_xor_si128(t0.m, t1.m);
+	t1p = vint8(astcenc_mm256_set_m128i(t1n, t1n));
+
+	__m128i t2n = _mm_xor_si128(t1.m, t2.m);
+	t2p = vint8(astcenc_mm256_set_m128i(t2n, t2n));
+
+	__m128i t3n = _mm_xor_si128(t2.m, t3.m);
+	t3p = vint8(astcenc_mm256_set_m128i(t3n, t3n));
+}
+
+/**
+ * @brief Perform an 8-bit 16-entry table lookup, with 32-bit indexes.
+ */
+ASTCENC_SIMD_INLINE vint8 vtable_8bt_32bi(vint8 t0, vint8 idx)
+{
+	// Set index byte MSB to 1 for unused bytes so shuffle returns zero
+	__m256i idxx = _mm256_or_si256(idx.m, _mm256_set1_epi32(static_cast<int>(0xFFFFFF00)));
+
+	__m256i result = _mm256_shuffle_epi8(t0.m, idxx);
+	return vint8(result);
+}
+
+/**
+ * @brief Perform an 8-bit 32-entry table lookup, with 32-bit indexes.
+ */
+ASTCENC_SIMD_INLINE vint8 vtable_8bt_32bi(vint8 t0, vint8 t1, vint8 idx)
+{
+	// Set index byte MSB to 1 for unused bytes so shuffle returns zero
+	__m256i idxx = _mm256_or_si256(idx.m, _mm256_set1_epi32(static_cast<int>(0xFFFFFF00)));
+
+	__m256i result = _mm256_shuffle_epi8(t0.m, idxx);
+	idxx = _mm256_sub_epi8(idxx, _mm256_set1_epi8(16));
+
+	__m256i result2 = _mm256_shuffle_epi8(t1.m, idxx);
+	result = _mm256_xor_si256(result, result2);
+	return vint8(result);
+}
+
+/**
+ * @brief Perform an 8-bit 64-entry table lookup, with 32-bit indexes.
+ */
+ASTCENC_SIMD_INLINE vint8 vtable_8bt_32bi(vint8 t0, vint8 t1, vint8 t2, vint8 t3, vint8 idx)
+{
+	// Set index byte MSB to 1 for unused bytes so shuffle returns zero
+	__m256i idxx = _mm256_or_si256(idx.m, _mm256_set1_epi32(static_cast<int>(0xFFFFFF00)));
+
+	__m256i result = _mm256_shuffle_epi8(t0.m, idxx);
+	idxx = _mm256_sub_epi8(idxx, _mm256_set1_epi8(16));
+
+	__m256i result2 = _mm256_shuffle_epi8(t1.m, idxx);
+	result = _mm256_xor_si256(result, result2);
+	idxx = _mm256_sub_epi8(idxx, _mm256_set1_epi8(16));
+
+	result2 = _mm256_shuffle_epi8(t2.m, idxx);
+	result = _mm256_xor_si256(result, result2);
+	idxx = _mm256_sub_epi8(idxx, _mm256_set1_epi8(16));
+
+	result2 = _mm256_shuffle_epi8(t3.m, idxx);
+	result = _mm256_xor_si256(result, result2);
+
+	return vint8(result);
+}
+
+/**
+ * @brief Return a vector of interleaved RGBA data.
+ *
+ * Input vectors have the value stored in the bottom 8 bits of each lane,
+ * with high  bits set to zero.
+ *
+ * Output vector stores a single RGBA texel packed in each lane.
+ */
+ASTCENC_SIMD_INLINE vint8 interleave_rgba8(vint8 r, vint8 g, vint8 b, vint8 a)
+{
+	return r + lsl<8>(g) + lsl<16>(b) + lsl<24>(a);
+}
+
+/**
+ * @brief Store a vector, skipping masked lanes.
+ *
+ * All masked lanes must be at the end of vector, after all non-masked lanes.
+ */
+ASTCENC_SIMD_INLINE void store_lanes_masked(int* base, vint8 data, vmask8 mask)
+{
+	_mm256_maskstore_epi32(base, _mm256_castps_si256(mask.m), data.m);
+}
+
+/**
+ * @brief Debug function to print a vector of ints.
+ */
+ASTCENC_SIMD_INLINE void print(vint8 a)
+{
+	alignas(ASTCENC_VECALIGN) int v[8];
+	storea(a, v);
+	printf("v8_i32:\n  %8d %8d %8d %8d %8d %8d %8d %8d\n",
+	       v[0], v[1], v[2], v[3], v[4], v[5], v[6], v[7]);
+}
+
+/**
+ * @brief Debug function to print a vector of ints.
+ */
+ASTCENC_SIMD_INLINE void printx(vint8 a)
+{
+	alignas(ASTCENC_VECALIGN) int v[8];
+	storea(a, v);
+	printf("v8_i32:\n  %08x %08x %08x %08x %08x %08x %08x %08x\n",
+	       v[0], v[1], v[2], v[3], v[4], v[5], v[6], v[7]);
+}
+
 /**
  * @brief Debug function to print a vector of floats.
  */
@@ -995,8 +1186,18 @@ ASTCENC_SIMD_INLINE void print(vfloat8 a)
 	alignas(ASTCENC_VECALIGN) float v[8];
 	storea(a, v);
 	printf("v8_f32:\n  %0.4f %0.4f %0.4f %0.4f %0.4f %0.4f %0.4f %0.4f\n",
-	       (double)v[0], (double)v[1], (double)v[2], (double)v[3],
-	       (double)v[4], (double)v[5], (double)v[6], (double)v[7]);
+	       static_cast<double>(v[0]), static_cast<double>(v[1]),
+	       static_cast<double>(v[2]), static_cast<double>(v[3]),
+	       static_cast<double>(v[4]), static_cast<double>(v[5]),
+	       static_cast<double>(v[6]), static_cast<double>(v[7]));
+}
+
+/**
+ * @brief Debug function to print a vector of masks.
+ */
+ASTCENC_SIMD_INLINE void print(vmask8 a)
+{
+	print(select(vint8(0), vint8(1), a));
 }
 
 #endif // #ifndef ASTC_VECMATHLIB_AVX2_8_H_INCLUDED
