@@ -1,6 +1,6 @@
 // SPDX-License-Identifier: Apache-2.0
 // ----------------------------------------------------------------------------
-// Copyright 2019-2021 Arm Limited
+// Copyright 2019-2022 Arm Limited
 //
 // Licensed under the Apache License, Version 2.0 (the "License"); you may not
 // use this file except in compliance with the License. You may obtain a copy
@@ -550,10 +550,15 @@ template <int s> ASTCENC_SIMD_INLINE vint4 lsl(vint4 a)
  */
 template <int s> ASTCENC_SIMD_INLINE vint4 lsr(vint4 a)
 {
-	return vint4((int)(((unsigned int)a.m[0]) >> s),
-	             (int)(((unsigned int)a.m[1]) >> s),
-	             (int)(((unsigned int)a.m[2]) >> s),
-	             (int)(((unsigned int)a.m[3]) >> s));
+	unsigned int as0 = static_cast<unsigned int>(a.m[0]) >> s;
+	unsigned int as1 = static_cast<unsigned int>(a.m[1]) >> s;
+	unsigned int as2 = static_cast<unsigned int>(a.m[2]) >> s;
+	unsigned int as3 = static_cast<unsigned int>(a.m[3]) >> s;
+
+	return vint4(static_cast<int>(as0),
+	             static_cast<int>(as1),
+	             static_cast<int>(as2),
+	             static_cast<int>(as3));
 }
 
 /**
@@ -644,7 +649,7 @@ ASTCENC_SIMD_INLINE void store(vint4 a, int* p)
  */
 ASTCENC_SIMD_INLINE void store_nbytes(vint4 a, uint8_t* p)
 {
-	int* pi = (int*)p;
+	int* pi = reinterpret_cast<int*>(p);
 	*pi = a.m[0];
 }
 
@@ -678,10 +683,10 @@ ASTCENC_SIMD_INLINE vint4 pack_low_bytes(vint4 a)
  */
 ASTCENC_SIMD_INLINE vint4 select(vint4 a, vint4 b, vmask4 cond)
 {
-	return vint4((cond.m[0] & 0x80000000) ? b.m[0] : a.m[0],
-	             (cond.m[1] & 0x80000000) ? b.m[1] : a.m[1],
-	             (cond.m[2] & 0x80000000) ? b.m[2] : a.m[2],
-	             (cond.m[3] & 0x80000000) ? b.m[3] : a.m[3]);
+	return vint4((cond.m[0] & static_cast<int>(0x80000000)) ? b.m[0] : a.m[0],
+	             (cond.m[1] & static_cast<int>(0x80000000)) ? b.m[1] : a.m[1],
+	             (cond.m[2] & static_cast<int>(0x80000000)) ? b.m[2] : a.m[2],
+	             (cond.m[3] & static_cast<int>(0x80000000)) ? b.m[3] : a.m[3]);
 }
 
 // ============================================================================
@@ -888,14 +893,25 @@ ASTCENC_SIMD_INLINE vfloat4 sqrt(vfloat4 a)
 }
 
 /**
- * @brief Return lanes from @c b if MSB of @c cond is set, else @c a.
+ * @brief Return lanes from @c b if @c cond is set, else @c a.
  */
 ASTCENC_SIMD_INLINE vfloat4 select(vfloat4 a, vfloat4 b, vmask4 cond)
 {
-	return vfloat4((cond.m[0] & 0x80000000) ? b.m[0] : a.m[0],
-	               (cond.m[1] & 0x80000000) ? b.m[1] : a.m[1],
-	               (cond.m[2] & 0x80000000) ? b.m[2] : a.m[2],
-	               (cond.m[3] & 0x80000000) ? b.m[3] : a.m[3]);
+	return vfloat4((cond.m[0] & static_cast<int>(0x80000000)) ? b.m[0] : a.m[0],
+	               (cond.m[1] & static_cast<int>(0x80000000)) ? b.m[1] : a.m[1],
+	               (cond.m[2] & static_cast<int>(0x80000000)) ? b.m[2] : a.m[2],
+	               (cond.m[3] & static_cast<int>(0x80000000)) ? b.m[3] : a.m[3]);
+}
+
+/**
+ * @brief Return lanes from @c b if MSB of @c cond is set, else @c a.
+ */
+ASTCENC_SIMD_INLINE vfloat4 select_msb(vfloat4 a, vfloat4 b, vmask4 cond)
+{
+	return vfloat4((cond.m[0] & static_cast<int>(0x80000000)) ? b.m[0] : a.m[0],
+	               (cond.m[1] & static_cast<int>(0x80000000)) ? b.m[1] : a.m[1],
+	               (cond.m[2] & static_cast<int>(0x80000000)) ? b.m[2] : a.m[2],
+	               (cond.m[3] & static_cast<int>(0x80000000)) ? b.m[3] : a.m[3]);
 }
 
 /**
@@ -936,10 +952,10 @@ ASTCENC_SIMD_INLINE void storea(vfloat4 a, float* ptr)
  */
 ASTCENC_SIMD_INLINE vint4 float_to_int(vfloat4 a)
 {
-	return vint4((int)a.m[0],
-	             (int)a.m[1],
-	             (int)a.m[2],
-	             (int)a.m[3]);
+	return vint4(static_cast<int>(a.m[0]),
+	             static_cast<int>(a.m[1]),
+	             static_cast<int>(a.m[2]),
+	             static_cast<int>(a.m[3]));
 }
 
 /**f
@@ -947,10 +963,10 @@ ASTCENC_SIMD_INLINE vint4 float_to_int(vfloat4 a)
  */
 ASTCENC_SIMD_INLINE vint4 float_to_int_rtn(vfloat4 a)
 {
-	return vint4((int)(a.m[0] + 0.5f),
-	             (int)(a.m[1] + 0.5f),
-	             (int)(a.m[2] + 0.5f),
-	             (int)(a.m[3] + 0.5f));
+	return vint4(static_cast<int>(a.m[0] + 0.5f),
+	             static_cast<int>(a.m[1] + 0.5f),
+	             static_cast<int>(a.m[2] + 0.5f),
+	             static_cast<int>(a.m[3] + 0.5f));
 }
 
 /**
@@ -958,10 +974,10 @@ ASTCENC_SIMD_INLINE vint4 float_to_int_rtn(vfloat4 a)
  */
 ASTCENC_SIMD_INLINE vfloat4 int_to_float(vint4 a)
 {
-	return vfloat4((float)a.m[0],
-	               (float)a.m[1],
-	               (float)a.m[2],
-	               (float)a.m[3]);
+	return vfloat4(static_cast<float>(a.m[0]),
+	               static_cast<float>(a.m[1]),
+	               static_cast<float>(a.m[2]),
+	               static_cast<float>(a.m[3]));
 }
 
 /**
@@ -990,10 +1006,10 @@ static inline uint16_t float_to_float16(float a)
 ASTCENC_SIMD_INLINE vfloat4 float16_to_float(vint4 a)
 {
 	return vfloat4(
-		sf16_to_float(a.lane<0>()),
-		sf16_to_float(a.lane<1>()),
-		sf16_to_float(a.lane<2>()),
-		sf16_to_float(a.lane<3>()));
+		sf16_to_float(static_cast<uint16_t>(a.lane<0>())),
+		sf16_to_float(static_cast<uint16_t>(a.lane<1>())),
+		sf16_to_float(static_cast<uint16_t>(a.lane<2>())),
+		sf16_to_float(static_cast<uint16_t>(a.lane<3>())));
 }
 
 /**
@@ -1032,4 +1048,122 @@ ASTCENC_SIMD_INLINE vfloat4 int_as_float(vint4 a)
 	return r;
 }
 
+/**
+ * @brief Prepare a vtable lookup table for use with the native SIMD size.
+ */
+ASTCENC_SIMD_INLINE void vtable_prepare(vint4 t0, vint4& t0p)
+{
+	t0p = t0;
+}
+
+/**
+ * @brief Prepare a vtable lookup table for use with the native SIMD size.
+ */
+ASTCENC_SIMD_INLINE void vtable_prepare(vint4 t0, vint4 t1, vint4& t0p, vint4& t1p)
+{
+	t0p = t0;
+	t1p = t1;
+}
+
+/**
+ * @brief Prepare a vtable lookup table for use with the native SIMD size.
+ */
+ASTCENC_SIMD_INLINE void vtable_prepare(
+	vint4 t0, vint4 t1, vint4 t2, vint4 t3,
+	vint4& t0p, vint4& t1p, vint4& t2p, vint4& t3p)
+{
+	t0p = t0;
+	t1p = t1;
+	t2p = t2;
+	t3p = t3;
+}
+
+/**
+ * @brief Perform an 8-bit 32-entry table lookup, with 32-bit indexes.
+ */
+ASTCENC_SIMD_INLINE vint4 vtable_8bt_32bi(vint4 t0, vint4 idx)
+{
+	uint8_t table[16];
+	storea(t0, reinterpret_cast<int*>(table +  0));
+
+	return vint4(table[idx.lane<0>()],
+	             table[idx.lane<1>()],
+	             table[idx.lane<2>()],
+	             table[idx.lane<3>()]);
+}
+
+
+/**
+ * @brief Perform an 8-bit 32-entry table lookup, with 32-bit indexes.
+ */
+ASTCENC_SIMD_INLINE vint4 vtable_8bt_32bi(vint4 t0, vint4 t1, vint4 idx)
+{
+	uint8_t table[32];
+	storea(t0, reinterpret_cast<int*>(table +  0));
+	storea(t1, reinterpret_cast<int*>(table + 16));
+
+	return vint4(table[idx.lane<0>()],
+	             table[idx.lane<1>()],
+	             table[idx.lane<2>()],
+	             table[idx.lane<3>()]);
+}
+
+/**
+ * @brief Perform an 8-bit 64-entry table lookup, with 32-bit indexes.
+ */
+ASTCENC_SIMD_INLINE vint4 vtable_8bt_32bi(vint4 t0, vint4 t1, vint4 t2, vint4 t3, vint4 idx)
+{
+	uint8_t table[64];
+	storea(t0, reinterpret_cast<int*>(table +  0));
+	storea(t1, reinterpret_cast<int*>(table + 16));
+	storea(t2, reinterpret_cast<int*>(table + 32));
+	storea(t3, reinterpret_cast<int*>(table + 48));
+
+	return vint4(table[idx.lane<0>()],
+	             table[idx.lane<1>()],
+	             table[idx.lane<2>()],
+	             table[idx.lane<3>()]);
+}
+
+/**
+ * @brief Return a vector of interleaved RGBA data.
+ *
+ * Input vectors have the value stored in the bottom 8 bits of each lane,
+ * with high  bits set to zero.
+ *
+ * Output vector stores a single RGBA texel packed in each lane.
+ */
+ASTCENC_SIMD_INLINE vint4 interleave_rgba8(vint4 r, vint4 g, vint4 b, vint4 a)
+{
+	return r + lsl<8>(g) + lsl<16>(b) + lsl<24>(a);
+}
+
+/**
+ * @brief Store a vector, skipping masked lanes.
+ *
+ * All masked lanes must be at the end of vector, after all non-masked lanes.
+ */
+ASTCENC_SIMD_INLINE void store_lanes_masked(int* base, vint4 data, vmask4 mask)
+{
+	if (mask.m[3])
+	{
+		store(data, base);
+	}
+	else if(mask.m[2])
+	{
+		base[0] = data.lane<0>();
+		base[1] = data.lane<1>();
+		base[2] = data.lane<2>();
+	}
+	else if(mask.m[1])
+	{
+		base[0] = data.lane<0>();
+		base[1] = data.lane<1>();
+	}
+	else if(mask.m[0])
+	{
+		base[0] = data.lane<0>();
+	}
+}
+
 #endif // #ifndef ASTC_VECMATHLIB_NONE_4_H_INCLUDED
