@@ -1,6 +1,6 @@
 // SPDX-License-Identifier: Apache-2.0
 // ----------------------------------------------------------------------------
-// Copyright 2011-2021 Arm Limited
+// Copyright 2011-2022 Arm Limited
 //
 // Licensed under the Apache License, Version 2.0 (the "License"); you may not
 // use this file except in compliance with the License. You may obtain a copy
@@ -44,51 +44,41 @@
 #include <cassert>
 #include <cstring>
 
-
-static constexpr unsigned int ANGULAR_STEPS { 40 };
-
-// Store a reduced sin/cos table for 64 possible weight values; this causes slight quality loss
-// compared to using sin() and cos() directly. Must be 2^N.
-static constexpr unsigned int SINCOS_STEPS { 64 };
+static constexpr unsigned int ANGULAR_STEPS { 32 };
 
 static_assert((ANGULAR_STEPS % ASTCENC_SIMD_WIDTH) == 0,
               "ANGULAR_STEPS must be multiple of ASTCENC_SIMD_WIDTH");
 
-static unsigned int max_angular_steps_needed_for_quant_level[13];
+static_assert(ANGULAR_STEPS >= 32,
+              "ANGULAR_STEPS must be at least max(steps_for_quant_level)");
 
-// The next-to-last entry is supposed to have the value 33. This because the 32-weight mode leaves a
-// double-sized hole in the middle of the weight space, so we are better off matching 33 weights.
-static const unsigned int quantization_steps_for_level[13] {
-	2, 3, 4, 5, 6, 8, 10, 12, 16, 20, 24, 33, 36
+// Store a reduced sin/cos table for 64 possible weight values; this causes
+// slight quality loss compared to using sin() and cos() directly. Must be 2^N.
+static constexpr unsigned int SINCOS_STEPS { 64 };
+
+static const uint8_t steps_for_quant_level[12] {
+	2, 3, 4, 5, 6, 8, 10, 12, 16, 20, 24, 32
 };
 
 alignas(ASTCENC_VECALIGN) static float sin_table[SINCOS_STEPS][ANGULAR_STEPS];
 alignas(ASTCENC_VECALIGN) static float cos_table[SINCOS_STEPS][ANGULAR_STEPS];
 
-#if !defined(NDEBUG)
+#if defined(ASTCENC_DIAGNOSTICS)
 	static bool print_once { true };
 #endif
 
 /* See header for documentation. */
 void prepare_angular_tables()
 {
-	unsigned int max_angular_steps_needed_for_quant_steps[ANGULAR_STEPS + 1];
 	for (unsigned int i = 0; i < ANGULAR_STEPS; i++)
 	{
-		float angle_step = (float)(i + 1);
+		float angle_step = static_cast<float>(i + 1);
 
 		for (unsigned int j = 0; j < SINCOS_STEPS; j++)
 		{
 			sin_table[j][i] = static_cast<float>(sinf((2.0f * astc::PI / (SINCOS_STEPS - 1.0f)) * angle_step * static_cast<float>(j)));
 			cos_table[j][i] = static_cast<float>(cosf((2.0f * astc::PI / (SINCOS_STEPS - 1.0f)) * angle_step * static_cast<float>(j)));
 		}
-
-		max_angular_steps_needed_for_quant_steps[i + 1] = astc::min(i + 1, ANGULAR_STEPS - 1);
-	}
-
-	for (unsigned int i = 0; i < 13; i++)
-	{
-		max_angular_steps_needed_for_quant_level[i] = max_angular_steps_needed_for_quant_steps[quantization_steps_for_level[i]];
 	}
 }
 
@@ -96,28 +86,26 @@ void prepare_angular_tables()
  * @brief Compute the angular alignment factors and offsets.
  *
  * @param      weight_count              The number of (decimated) weights.
- * @param      dec_weight_quant_uvalue   The decimated and quantized weight values.
- * @param      dec_weight_quant_sig      The significance of each weight.
+ * @param      dec_weight_ideal_value    The ideal decimated unquantized weight values.
  * @param      max_angular_steps         The maximum number of steps to be tested.
  * @param[out] offsets                   The output angular offsets array.
  */
 static void compute_angular_offsets(
 	unsigned int weight_count,
-	const float* dec_weight_quant_uvalue,
-	const float* dec_weight_quant_sig,
+	const float* dec_weight_ideal_value,
 	unsigned int max_angular_steps,
 	float* offsets
 ) {
 	promise(weight_count > 0);
 	promise(max_angular_steps > 0);
 
-	alignas(ASTCENC_VECALIGN) int isamplev[BLOCK_MAX_WEIGHTS] { 0 };
+	alignas(ASTCENC_VECALIGN) int isamplev[BLOCK_MAX_WEIGHTS];
 
 	// Precompute isample; arrays are always allocated 64 elements long
 	for (unsigned int i = 0; i < weight_count; i += ASTCENC_SIMD_WIDTH)
 	{
 		// Add 2^23 and interpreting bits extracts round-to-nearest int
-		vfloat sample = loada(dec_weight_quant_uvalue + i) * (SINCOS_STEPS - 1.0f) + vfloat(12582912.0f);
+		vfloat sample = loada(dec_weight_ideal_value + i) * (SINCOS_STEPS - 1.0f) + vfloat(12582912.0f);
 		vint isample = float_as_int(sample) & vint((SINCOS_STEPS - 1));
 		storea(isample, isamplev + i);
 	}
@@ -133,9 +121,8 @@ static void compute_angular_offsets(
 		for (unsigned int j = 0; j < weight_count; j++)
 		{
 			int isample = isamplev[j];
-			vfloat sample_weightv(dec_weight_quant_sig[j]);
-			anglesum_x += loada(cos_table[isample] + i) * sample_weightv;
-			anglesum_y += loada(sin_table[isample] + i) * sample_weightv;
+			anglesum_x += loada(cos_table[isample] + i);
+			anglesum_y += loada(sin_table[isample] + i);
 		}
 
 		vfloat angle = atan2(anglesum_y, anglesum_x);
@@ -152,8 +139,7 @@ static void compute_angular_offsets(
  * forcing samples that should have had one weight value one step up or down.
  *
  * @param      weight_count              The number of (decimated) weights.
- * @param      dec_weight_quant_uvalue   The decimated and quantized weight values.
- * @param      dec_weight_quant_sig      The significance of each weight.
+ * @param      dec_weight_ideal_value    The ideal decimated unquantized weight values.
  * @param      max_angular_steps         The maximum number of steps to be tested.
  * @param      max_quant_steps           The maximum quantization level to be tested.
  * @param      offsets                   The angular offsets array.
@@ -165,12 +151,11 @@ static void compute_angular_offsets(
  */
 static void compute_lowest_and_highest_weight(
 	unsigned int weight_count,
-	const float* dec_weight_quant_uvalue,
-	const float* dec_weight_quant_sig,
+	const float* dec_weight_ideal_value,
 	unsigned int max_angular_steps,
 	unsigned int max_quant_steps,
 	const float* offsets,
-	int* lowest_weight,
+	float* lowest_weight,
 	int* weight_span,
 	float* error,
 	float* cut_low_weight_error,
@@ -189,16 +174,14 @@ static void compute_lowest_and_highest_weight(
 		vfloat errval = vfloat::zero();
 		vfloat cut_low_weight_err = vfloat::zero();
 		vfloat cut_high_weight_err = vfloat::zero();
-		vfloat offset = loada(&offsets[sp]);
+		vfloat offset = loada(offsets + sp);
 
 		for (unsigned int j = 0; j < weight_count; ++j)
 		{
-			vfloat wt = load1(&dec_weight_quant_sig[j]);
-			vfloat sval = load1(&dec_weight_quant_uvalue[j]) * rcp_stepsize - offset;
+			vfloat sval = load1(dec_weight_ideal_value + j) * rcp_stepsize - offset;
 			vfloat svalrte = round(sval);
-			vfloat dif = sval - svalrte;
-			vfloat dwt = dif * wt;
-			errval += dwt * dif;
+			vfloat diff = sval - svalrte;
+			errval += diff * diff;
 
 			// Reset tracker on min hit
 			vmask mask = svalrte < minidx;
@@ -207,7 +190,7 @@ static void compute_lowest_and_highest_weight(
 
 			// Accumulate on min hit
 			mask = svalrte == minidx;
-			vfloat accum = cut_low_weight_err + wt - vfloat(2.0f) * dwt;
+			vfloat accum = cut_low_weight_err + vfloat(1.0f) - vfloat(2.0f) * diff;
 			cut_low_weight_err = select(cut_low_weight_err, accum, mask);
 
 			// Reset tracker on max hit
@@ -217,7 +200,7 @@ static void compute_lowest_and_highest_weight(
 
 			// Accumulate on max hit
 			mask = svalrte == maxidx;
-			accum = cut_high_weight_err + wt + vfloat(2.0f) * dwt;
+			accum = cut_high_weight_err + vfloat(1.0f) + vfloat(2.0f) * diff;
 			cut_high_weight_err = select(cut_high_weight_err, accum, mask);
 		}
 
@@ -225,16 +208,16 @@ static void compute_lowest_and_highest_weight(
 		vint span = float_to_int(maxidx - minidx + vfloat(1));
 		span = min(span, vint(max_quant_steps + 3));
 		span = max(span, vint(2));
-		storea(float_to_int(minidx), &lowest_weight[sp]);
-		storea(span, &weight_span[sp]);
+		storea(minidx, lowest_weight + sp);
+		storea(span, weight_span + sp);
 
 		// The cut_(lowest/highest)_weight_error indicate the error that results from  forcing
 		// samples that should have had the weight value one step (up/down).
 		vfloat ssize = 1.0f / rcp_stepsize;
 		vfloat errscale = ssize * ssize;
-		storea(errval * errscale, &error[sp]);
-		storea(cut_low_weight_err * errscale, &cut_low_weight_error[sp]);
-		storea(cut_high_weight_err * errscale, &cut_high_weight_error[sp]);
+		storea(errval * errscale, error + sp);
+		storea(cut_low_weight_err * errscale, cut_low_weight_error + sp);
+		storea(cut_high_weight_err * errscale, cut_high_weight_error + sp);
 
 		rcp_stepsize = rcp_stepsize + vfloat(ASTCENC_SIMD_WIDTH);
 	}
@@ -244,34 +227,33 @@ static void compute_lowest_and_highest_weight(
  * @brief The main function for the angular algorithm.
  *
  * @param      weight_count              The number of (decimated) weights.
- * @param      dec_weight_quant_uvalue   The decimated and quantized weight value.
- * @param      dec_weight_quant_sig      The significance of each weight.
+ * @param      dec_weight_ideal_value    The ideal decimated unquantized weight values.
  * @param      max_quant_level           The maximum quantization level to be tested.
  * @param[out] low_value                 Per angular step, the lowest weight value.
  * @param[out] high_value                Per angular step, the highest weight value.
  */
 static void compute_angular_endpoints_for_quant_levels(
 	unsigned int weight_count,
-	const float* dec_weight_quant_uvalue,
-	const float* dec_weight_quant_sig,
+	const float* dec_weight_ideal_value,
 	unsigned int max_quant_level,
-	float low_value[12],
-	float high_value[12]
+	float low_value[TUNE_MAX_ANGULAR_QUANT + 1],
+	float high_value[TUNE_MAX_ANGULAR_QUANT + 1]
 ) {
-	unsigned int max_quant_steps = quantization_steps_for_level[max_quant_level];
+	unsigned int max_quant_steps = steps_for_quant_level[max_quant_level];
+	unsigned int max_angular_steps = steps_for_quant_level[max_quant_level];
 
 	alignas(ASTCENC_VECALIGN) float angular_offsets[ANGULAR_STEPS];
-	unsigned int max_angular_steps = max_angular_steps_needed_for_quant_level[max_quant_level];
-	compute_angular_offsets(weight_count, dec_weight_quant_uvalue, dec_weight_quant_sig,
+
+	compute_angular_offsets(weight_count, dec_weight_ideal_value,
 	                        max_angular_steps, angular_offsets);
 
-	alignas(ASTCENC_VECALIGN) int32_t lowest_weight[ANGULAR_STEPS];
+	alignas(ASTCENC_VECALIGN) float lowest_weight[ANGULAR_STEPS];
 	alignas(ASTCENC_VECALIGN) int32_t weight_span[ANGULAR_STEPS];
 	alignas(ASTCENC_VECALIGN) float error[ANGULAR_STEPS];
 	alignas(ASTCENC_VECALIGN) float cut_low_weight_error[ANGULAR_STEPS];
 	alignas(ASTCENC_VECALIGN) float cut_high_weight_error[ANGULAR_STEPS];
 
-	compute_lowest_and_highest_weight(weight_count, dec_weight_quant_uvalue, dec_weight_quant_sig,
+	compute_lowest_and_highest_weight(weight_count, dec_weight_ideal_value,
 	                                  max_angular_steps, max_quant_steps,
 	                                  angular_offsets, lowest_weight, weight_span, error,
 	                                  cut_low_weight_error, cut_high_weight_error);
@@ -279,7 +261,7 @@ static void compute_angular_endpoints_for_quant_levels(
 	// For each quantization level, find the best error terms. Use packed vectors so data-dependent
 	// branches can become selects. This involves some integer to float casts, but the values are
 	// small enough so they never round the wrong way.
-	vfloat4 best_results[40];
+	vfloat4 best_results[36];
 
 	// Initialize the array to some safe defaults
 	promise(max_quant_steps > 0);
@@ -294,42 +276,45 @@ static void compute_angular_endpoints_for_quant_levels(
 	promise(max_angular_steps > 0);
 	for (unsigned int i = 0; i < max_angular_steps; i++)
 	{
+		float i_flt = static_cast<float>(i);
+
 		int idx_span = weight_span[i];
+
 		float error_cut_low = error[i] + cut_low_weight_error[i];
 		float error_cut_high = error[i] + cut_high_weight_error[i];
 		float error_cut_low_high = error[i] + cut_low_weight_error[i] + cut_high_weight_error[i];
 
 		// Check best error against record N
 		vfloat4 best_result = best_results[idx_span];
-		vfloat4 new_result = vfloat4(error[i], (float)i, 0.0f, 0.0f);
+		vfloat4 new_result = vfloat4(error[i], i_flt, 0.0f, 0.0f);
 		vmask4 mask1(best_result.lane<0>() > error[i]);
 		best_results[idx_span] = select(best_result, new_result, mask1);
 
 		// Check best error against record N-1 with either cut low or cut high
 		best_result = best_results[idx_span - 1];
 
-		new_result = vfloat4(error_cut_low, (float)i, 1.0f, 0.0f);
+		new_result = vfloat4(error_cut_low, i_flt, 1.0f, 0.0f);
 		vmask4 mask2(best_result.lane<0>() > error_cut_low);
 		best_result = select(best_result, new_result, mask2);
 
-		new_result = vfloat4(error_cut_high, (float)i, 0.0f, 0.0f);
+		new_result = vfloat4(error_cut_high, i_flt, 0.0f, 0.0f);
 		vmask4 mask3(best_result.lane<0>() > error_cut_high);
 		best_results[idx_span - 1] = select(best_result, new_result, mask3);
 
 		// Check best error against record N-2 with both cut low and high
 		best_result = best_results[idx_span - 2];
-		new_result = vfloat4(error_cut_low_high, (float)i, 1.0f, 0.0f);
+		new_result = vfloat4(error_cut_low_high, i_flt, 1.0f, 0.0f);
 		vmask4 mask4(best_result.lane<0>() > error_cut_low_high);
 		best_results[idx_span - 2] = select(best_result, new_result, mask4);
 	}
 
 	for (unsigned int i = 0; i <= max_quant_level; i++)
 	{
-		unsigned int q = quantization_steps_for_level[i];
-		int bsi = (int)best_results[q].lane<1>();
+		unsigned int q = steps_for_quant_level[i];
+		int bsi = static_cast<int>(best_results[q].lane<1>());
 
 		// Did we find anything?
-#if !defined(NDEBUG)
+#if defined(ASTCENC_DIAGNOSTICS)
 		if ((bsi < 0) && print_once)
 		{
 			print_once = false;
@@ -339,13 +324,12 @@ static void compute_angular_endpoints_for_quant_levels(
 
 		bsi = astc::max(0, bsi);
 
-		float stepsize = 1.0f / (1.0f + (float)bsi);
-		int lwi = lowest_weight[bsi] + (int)best_results[q].lane<2>();
-		int hwi = lwi + q - 1;
+		float lwi = lowest_weight[bsi] + best_results[q].lane<2>();
+		float hwi = lwi + static_cast<float>(q) - 1.0f;
 
-		float offset = angular_offsets[bsi] * stepsize;
-		low_value[i] = offset + static_cast<float>(lwi) * stepsize;
-		high_value[i] = offset + static_cast<float>(hwi) * stepsize;
+		float stepsize = 1.0f / (1.0f + static_cast<float>(bsi));
+		low_value[i]  = (angular_offsets[bsi] + lwi) * stepsize;
+		high_value[i] = (angular_offsets[bsi] + hwi) * stepsize;
 	}
 }
 
@@ -358,7 +342,6 @@ static void compute_angular_endpoints_for_quant_levels(
  *
  * @param      weight_count              The number of (decimated) weights.
  * @param      dec_weight_quant_uvalue   The decimated and quantized weight values.
- * @param      dec_weight_quant_sig      The significance of each weight.
  * @param      max_angular_steps         The maximum number of steps to be tested.
  * @param      max_quant_steps           The maximum quantization level to be tested.
  * @param      offsets                   The angular offsets array.
@@ -369,11 +352,10 @@ static void compute_angular_endpoints_for_quant_levels(
 static void compute_lowest_and_highest_weight_lwc(
 	unsigned int weight_count,
 	const float* dec_weight_quant_uvalue,
-	const float* dec_weight_quant_sig,
 	unsigned int max_angular_steps,
 	unsigned int max_quant_steps,
 	const float* offsets,
-	int* lowest_weight,
+	float* lowest_weight,
 	int* weight_span,
 	float* error
 ) {
@@ -388,38 +370,32 @@ static void compute_lowest_and_highest_weight_lwc(
 		vfloat minidx(128.0f);
 		vfloat maxidx(-128.0f);
 		vfloat errval = vfloat::zero();
-		vfloat offset = loada(&offsets[sp]);
+		vfloat offset = loada(offsets + sp);
 
 		for (unsigned int j = 0; j < weight_count; ++j)
 		{
-			vfloat wt = load1(&dec_weight_quant_sig[j]);
-			vfloat sval = load1(&dec_weight_quant_uvalue[j]) * rcp_stepsize - offset;
+			vfloat sval = load1(dec_weight_quant_uvalue + j) * rcp_stepsize - offset;
 			vfloat svalrte = round(sval);
-			vfloat dif = sval - svalrte;
-			vfloat dwt = dif * wt;
-			errval += dwt * dif;
+			vfloat diff = sval - svalrte;
+			errval += diff * diff;
 
-			// Reset tracker on min hit
-			vmask mask = svalrte < minidx;
-			minidx = select(minidx, svalrte, mask);
-
-			// Reset tracker on max hit
-			mask = svalrte > maxidx;
-			maxidx = select(maxidx, svalrte, mask);
+			// Compute min and max quantized weight spans for each step
+			minidx = min(minidx, svalrte);
+			maxidx = max(maxidx, svalrte);
 		}
 
 		// Write out min weight and weight span; clamp span to a usable range
 		vint span = float_to_int(maxidx - minidx + vfloat(1.0f));
 		span = min(span, vint(max_quant_steps + 3));
 		span = max(span, vint(2));
-		storea(float_to_int(minidx), &lowest_weight[sp]);
-		storea(span, &weight_span[sp]);
+		storea(minidx, lowest_weight + sp);
+		storea(span, weight_span + sp);
 
-		// The cut_(lowest/highest)_weight_error indicate the error that results from  forcing
-		// samples that should have had the weight value one step (up/down).
+		// The cut_(lowest/highest)_weight_error indicate the error that results from
+		// forcing samples that should have had the weight value one step (up/down).
 		vfloat ssize = 1.0f / rcp_stepsize;
 		vfloat errscale = ssize * ssize;
-		storea(errval * errscale, &error[sp]);
+		storea(errval * errscale, error + sp);
 
 		rcp_stepsize = rcp_stepsize + vfloat(ASTCENC_SIMD_WIDTH);
 	}
@@ -429,48 +405,44 @@ static void compute_lowest_and_highest_weight_lwc(
  * @brief The main function for the angular algorithm, variant for low weight count.
  *
  * @param      weight_count              The number of (decimated) weights.
- * @param      dec_weight_quant_uvalue   The decimated and quantized weight value.
- * @param      dec_weight_quant_sig      The significance of each weight.
+ * @param      dec_weight_ideal_value    The ideal decimated unquantized weight values.
  * @param      max_quant_level           The maximum quantization level to be tested.
  * @param[out] low_value                 Per angular step, the lowest weight value.
  * @param[out] high_value                Per angular step, the highest weight value.
  */
 static void compute_angular_endpoints_for_quant_levels_lwc(
 	unsigned int weight_count,
-	const float* dec_weight_quant_uvalue,
-	const float* dec_weight_quant_sig,
+	const float* dec_weight_ideal_value,
 	unsigned int max_quant_level,
-	float low_value[12],
-	float high_value[12]
+	float low_value[TUNE_MAX_ANGULAR_QUANT + 1],
+	float high_value[TUNE_MAX_ANGULAR_QUANT + 1]
 ) {
-	unsigned int max_quant_steps = quantization_steps_for_level[max_quant_level];
-	unsigned int max_angular_steps = max_angular_steps_needed_for_quant_level[max_quant_level];
+	unsigned int max_quant_steps = steps_for_quant_level[max_quant_level];
+	unsigned int max_angular_steps = steps_for_quant_level[max_quant_level];
 
 	alignas(ASTCENC_VECALIGN) float angular_offsets[ANGULAR_STEPS];
-	alignas(ASTCENC_VECALIGN) int32_t lowest_weight[ANGULAR_STEPS];
+	alignas(ASTCENC_VECALIGN) float lowest_weight[ANGULAR_STEPS];
 	alignas(ASTCENC_VECALIGN) int32_t weight_span[ANGULAR_STEPS];
 	alignas(ASTCENC_VECALIGN) float error[ANGULAR_STEPS];
 
-	compute_angular_offsets(weight_count, dec_weight_quant_uvalue, dec_weight_quant_sig,
+	compute_angular_offsets(weight_count, dec_weight_ideal_value,
 	                        max_angular_steps, angular_offsets);
 
 
-	compute_lowest_and_highest_weight_lwc(weight_count, dec_weight_quant_uvalue, dec_weight_quant_sig,
+	compute_lowest_and_highest_weight_lwc(weight_count, dec_weight_ideal_value,
 	                                      max_angular_steps, max_quant_steps,
 	                                      angular_offsets, lowest_weight, weight_span, error);
 
 	// For each quantization level, find the best error terms. Use packed vectors so data-dependent
 	// branches can become selects. This involves some integer to float casts, but the values are
 	// small enough so they never round the wrong way.
-	float best_error[ANGULAR_STEPS];
-	int best_index[ANGULAR_STEPS];
+	vfloat4 best_results[36];
 
 	// Initialize the array to some safe defaults
 	promise(max_quant_steps > 0);
 	for (unsigned int i = 0; i < (max_quant_steps + 4); i++)
 	{
-		best_error[i] = ERROR_CALC_DEFAULT;
-		best_index[i] = -1;
+		best_results[i] = vfloat4(ERROR_CALC_DEFAULT, -1.0f, 0.0f, 0.0f);
 	}
 
 	promise(max_angular_steps > 0);
@@ -479,21 +451,19 @@ static void compute_angular_endpoints_for_quant_levels_lwc(
 		int idx_span = weight_span[i];
 
 		// Check best error against record N
-		float current_best = best_error[idx_span];
-		if (error[i] < current_best)
-		{
-			best_error[idx_span] = error[i];
-			best_index[idx_span] = i;
-		}
+		vfloat4 current_best = best_results[idx_span];
+		vfloat4 candidate = vfloat4(error[i], static_cast<float>(i), 0.0f, 0.0f);
+		vmask4 mask(current_best.lane<0>() > error[i]);
+		best_results[idx_span] = select(current_best, candidate, mask);
 	}
 
 	for (unsigned int i = 0; i <= max_quant_level; i++)
 	{
-		unsigned int q = quantization_steps_for_level[i];
-		int bsi = best_index[q];
+		unsigned int q = steps_for_quant_level[i];
+		int bsi = static_cast<int>(best_results[q].lane<1>());
 
 		// Did we find anything?
-#if !defined(NDEBUG)
+#if defined(ASTCENC_DIAGNOSTICS)
 		if ((bsi < 0) && print_once)
 		{
 			print_once = false;
@@ -503,11 +473,12 @@ static void compute_angular_endpoints_for_quant_levels_lwc(
 
 		bsi = astc::max(0, bsi);
 
-		int lwi = lowest_weight[bsi];
-		int hwi = lwi + q - 1;
+		float lwi = lowest_weight[bsi];
+		float hwi = lwi + static_cast<float>(q) - 1.0f;
 
-		low_value[i]  = (angular_offsets[bsi] + static_cast<float>(lwi)) / (1.0f + (float)bsi);
-		high_value[i] = (angular_offsets[bsi] + static_cast<float>(hwi)) / (1.0f + (float)bsi);
+		float stepsize = 1.0f / (1.0f + static_cast<float>(bsi));
+		low_value[i]  = (angular_offsets[bsi] + lwi) * stepsize;
+		high_value[i] = (angular_offsets[bsi] + hwi) * stepsize;
 	}
 }
 
@@ -516,61 +487,77 @@ void compute_angular_endpoints_1plane(
 	unsigned int tune_low_weight_limit,
 	bool only_always,
 	const block_size_descriptor& bsd,
-	const float* dec_weight_quant_uvalue,
-	const float* dec_weight_quant_sig,
-	float low_value[WEIGHTS_MAX_BLOCK_MODES],
-	float high_value[WEIGHTS_MAX_BLOCK_MODES]
+	const float* dec_weight_ideal_value,
+	unsigned int max_weight_quant,
+	compression_working_buffers& tmpbuf
 ) {
-	float low_values[WEIGHTS_MAX_DECIMATION_MODES][12];
-	float high_values[WEIGHTS_MAX_DECIMATION_MODES][12];
+	float (&low_value)[WEIGHTS_MAX_BLOCK_MODES] = tmpbuf.weight_low_value1;
+	float (&high_value)[WEIGHTS_MAX_BLOCK_MODES] = tmpbuf.weight_high_value1;
 
-	unsigned int max_decimation_modes = only_always ? bsd.always_decimation_mode_count
-	                                                : bsd.decimation_mode_count;
+	float (&low_values)[WEIGHTS_MAX_DECIMATION_MODES][TUNE_MAX_ANGULAR_QUANT + 1] = tmpbuf.weight_low_values1;
+	float (&high_values)[WEIGHTS_MAX_DECIMATION_MODES][TUNE_MAX_ANGULAR_QUANT + 1] = tmpbuf.weight_high_values1;
+
+	unsigned int max_decimation_modes = only_always ? bsd.decimation_mode_count_always
+	                                                : bsd.decimation_mode_count_selected;
 	promise(max_decimation_modes > 0);
 	for (unsigned int i = 0; i < max_decimation_modes; i++)
 	{
 		const decimation_mode& dm = bsd.decimation_modes[i];
-		if (dm.maxprec_1plane < 0 || !dm.percentile_hit)
+		if (!dm.is_ref_1_plane(static_cast<quant_method>(max_weight_quant)))
 		{
 			continue;
 		}
 
-		unsigned int weight_count = bsd.decimation_tables[i]->weight_count;
+		unsigned int weight_count = bsd.get_decimation_info(i).weight_count;
+
+		unsigned int max_precision = dm.maxprec_1plane;
+		if (max_precision > TUNE_MAX_ANGULAR_QUANT)
+		{
+			max_precision = TUNE_MAX_ANGULAR_QUANT;
+		}
+
+		if (max_precision > max_weight_quant)
+		{
+			max_precision = max_weight_quant;
+		}
 
 		if (weight_count < tune_low_weight_limit)
 		{
 			compute_angular_endpoints_for_quant_levels_lwc(
 				weight_count,
-				dec_weight_quant_uvalue + i * BLOCK_MAX_WEIGHTS,
-				dec_weight_quant_sig + i * BLOCK_MAX_WEIGHTS,
-				dm.maxprec_1plane, low_values[i], high_values[i]);
+				dec_weight_ideal_value + i * BLOCK_MAX_WEIGHTS,
+				max_precision, low_values[i], high_values[i]);
 		}
 		else
 		{
 			compute_angular_endpoints_for_quant_levels(
 				weight_count,
-				dec_weight_quant_uvalue + i * BLOCK_MAX_WEIGHTS,
-				dec_weight_quant_sig + i * BLOCK_MAX_WEIGHTS,
-				dm.maxprec_1plane, low_values[i], high_values[i]);
+				dec_weight_ideal_value + i * BLOCK_MAX_WEIGHTS,
+				max_precision, low_values[i], high_values[i]);
 		}
 	}
 
-	unsigned int max_block_modes = only_always ? bsd.always_block_mode_count
-	                                           : bsd.block_mode_count;
+	unsigned int max_block_modes = only_always ? bsd.block_mode_count_1plane_always
+	                                           : bsd.block_mode_count_1plane_selected;
 	promise(max_block_modes > 0);
 	for (unsigned int i = 0; i < max_block_modes; ++i)
 	{
 		const block_mode& bm = bsd.block_modes[i];
-		if (bm.is_dual_plane || !bm.percentile_hit)
-		{
-			continue;
-		}
+		assert(!bm.is_dual_plane);
 
 		unsigned int quant_mode = bm.quant_mode;
 		unsigned int decim_mode = bm.decimation_mode;
 
-		low_value[i] = low_values[decim_mode][quant_mode];
-		high_value[i] = high_values[decim_mode][quant_mode];
+		if (quant_mode <= TUNE_MAX_ANGULAR_QUANT)
+		{
+			low_value[i] = low_values[decim_mode][quant_mode];
+			high_value[i] = high_values[decim_mode][quant_mode];
+		}
+		else
+		{
+			low_value[i] = 0.0f;
+			high_value[i] = 1.0f;
+		}
 	}
 }
 
@@ -578,75 +565,90 @@ void compute_angular_endpoints_1plane(
 void compute_angular_endpoints_2planes(
 	unsigned int tune_low_weight_limit,
 	const block_size_descriptor& bsd,
-	const float* dec_weight_quant_uvalue,
-	const float* dec_weight_quant_sig,
-	float low_value1[WEIGHTS_MAX_BLOCK_MODES],
-	float high_value1[WEIGHTS_MAX_BLOCK_MODES],
-	float low_value2[WEIGHTS_MAX_BLOCK_MODES],
-	float high_value2[WEIGHTS_MAX_BLOCK_MODES]
+	const float* dec_weight_ideal_value,
+	unsigned int max_weight_quant,
+	compression_working_buffers& tmpbuf
 ) {
-	float low_values1[WEIGHTS_MAX_DECIMATION_MODES][12];
-	float high_values1[WEIGHTS_MAX_DECIMATION_MODES][12];
-	float low_values2[WEIGHTS_MAX_DECIMATION_MODES][12];
-	float high_values2[WEIGHTS_MAX_DECIMATION_MODES][12];
+	float (&low_value1)[WEIGHTS_MAX_BLOCK_MODES] = tmpbuf.weight_low_value1;
+	float (&high_value1)[WEIGHTS_MAX_BLOCK_MODES] = tmpbuf.weight_high_value1;
+	float (&low_value2)[WEIGHTS_MAX_BLOCK_MODES] = tmpbuf.weight_low_value2;
+	float (&high_value2)[WEIGHTS_MAX_BLOCK_MODES] = tmpbuf.weight_high_value2;
 
-	promise(bsd.decimation_mode_count > 0);
-	for (unsigned int i = 0; i < bsd.decimation_mode_count; i++)
+	float (&low_values1)[WEIGHTS_MAX_DECIMATION_MODES][TUNE_MAX_ANGULAR_QUANT + 1] = tmpbuf.weight_low_values1;
+	float (&high_values1)[WEIGHTS_MAX_DECIMATION_MODES][TUNE_MAX_ANGULAR_QUANT + 1] = tmpbuf.weight_high_values1;
+	float (&low_values2)[WEIGHTS_MAX_DECIMATION_MODES][TUNE_MAX_ANGULAR_QUANT + 1] = tmpbuf.weight_low_values2;
+	float (&high_values2)[WEIGHTS_MAX_DECIMATION_MODES][TUNE_MAX_ANGULAR_QUANT + 1] = tmpbuf.weight_high_values2;
+
+	promise(bsd.decimation_mode_count_selected > 0);
+	for (unsigned int i = 0; i < bsd.decimation_mode_count_selected; i++)
 	{
 		const decimation_mode& dm = bsd.decimation_modes[i];
-		if (dm.maxprec_2planes < 0 || !dm.percentile_hit)
+		if (!dm.is_ref_2_plane(static_cast<quant_method>(max_weight_quant)))
 		{
 			continue;
 		}
 
-		unsigned int weight_count = bsd.decimation_tables[i]->weight_count;
+		unsigned int weight_count = bsd.get_decimation_info(i).weight_count;
+
+		unsigned int max_precision = dm.maxprec_2planes;
+		if (max_precision > TUNE_MAX_ANGULAR_QUANT)
+		{
+			max_precision = TUNE_MAX_ANGULAR_QUANT;
+		}
+
+		if (max_precision > max_weight_quant)
+		{
+			max_precision = max_weight_quant;
+		}
 
 		if (weight_count < tune_low_weight_limit)
 		{
 			compute_angular_endpoints_for_quant_levels_lwc(
 				weight_count,
-				dec_weight_quant_uvalue + i * BLOCK_MAX_WEIGHTS,
-				dec_weight_quant_sig + i * BLOCK_MAX_WEIGHTS,
-				dm.maxprec_2planes, low_values1[i], high_values1[i]);
+				dec_weight_ideal_value + i * BLOCK_MAX_WEIGHTS,
+				max_precision, low_values1[i], high_values1[i]);
 
 			compute_angular_endpoints_for_quant_levels_lwc(
 				weight_count,
-				dec_weight_quant_uvalue + i * BLOCK_MAX_WEIGHTS + WEIGHTS_PLANE2_OFFSET,
-				dec_weight_quant_sig + i * BLOCK_MAX_WEIGHTS + WEIGHTS_PLANE2_OFFSET,
-				dm.maxprec_2planes, low_values2[i], high_values2[i]);
+				dec_weight_ideal_value + i * BLOCK_MAX_WEIGHTS + WEIGHTS_PLANE2_OFFSET,
+				max_precision, low_values2[i], high_values2[i]);
 		}
 		else
 		{
 			compute_angular_endpoints_for_quant_levels(
 				weight_count,
-				dec_weight_quant_uvalue + i * BLOCK_MAX_WEIGHTS,
-				dec_weight_quant_sig + i * BLOCK_MAX_WEIGHTS,
-				dm.maxprec_2planes, low_values1[i], high_values1[i]);
+				dec_weight_ideal_value + i * BLOCK_MAX_WEIGHTS,
+				max_precision, low_values1[i], high_values1[i]);
 
 			compute_angular_endpoints_for_quant_levels(
 				weight_count,
-				dec_weight_quant_uvalue + i * BLOCK_MAX_WEIGHTS + WEIGHTS_PLANE2_OFFSET,
-				dec_weight_quant_sig + i * BLOCK_MAX_WEIGHTS + WEIGHTS_PLANE2_OFFSET,
-				dm.maxprec_2planes, low_values2[i], high_values2[i]);
+				dec_weight_ideal_value + i * BLOCK_MAX_WEIGHTS + WEIGHTS_PLANE2_OFFSET,
+				max_precision, low_values2[i], high_values2[i]);
 		}
 	}
 
-	promise(bsd.block_mode_count > 0);
-	for (unsigned int i = 0; i < bsd.block_mode_count; ++i)
+	unsigned int start = bsd.block_mode_count_1plane_selected;
+	unsigned int end = bsd.block_mode_count_1plane_2plane_selected;
+	for (unsigned int i = start; i < end; i++)
 	{
 		const block_mode& bm = bsd.block_modes[i];
-		if (!bm.is_dual_plane || !bm.percentile_hit)
-		{
-			continue;
-		}
-
 		unsigned int quant_mode = bm.quant_mode;
 		unsigned int decim_mode = bm.decimation_mode;
 
-		low_value1[i] = low_values1[decim_mode][quant_mode];
-		high_value1[i] = high_values1[decim_mode][quant_mode];
-		low_value2[i] = low_values2[decim_mode][quant_mode];
-		high_value2[i] = high_values2[decim_mode][quant_mode];
+		if (quant_mode <= TUNE_MAX_ANGULAR_QUANT)
+		{
+			low_value1[i] = low_values1[decim_mode][quant_mode];
+			high_value1[i] = high_values1[decim_mode][quant_mode];
+			low_value2[i] = low_values2[decim_mode][quant_mode];
+			high_value2[i] = high_values2[decim_mode][quant_mode];
+		}
+		else
+		{
+			low_value1[i] = 0.0f;
+			high_value1[i] = 1.0f;
+			low_value2[i] = 0.0f;
+			high_value2[i] = 1.0f;
+		}
 	}
 }
 
