@@ -1,6 +1,6 @@
 // SPDX-License-Identifier: Apache-2.0
 // ----------------------------------------------------------------------------
-// Copyright 2011-2021 Arm Limited
+// Copyright 2011-2022 Arm Limited
 //
 // Licensed under the Apache License, Version 2.0 (the "License"); you may not
 // use this file except in compliance with the License. You may obtain a copy
@@ -24,9 +24,11 @@
 
 #include <cassert>
 #include <cstring>
+#include <functional>
 #include <string>
 #include <sstream>
 #include <vector>
+#include <memory>
 
 /* ============================================================================
 	Data structure definitions
@@ -655,37 +657,6 @@ static int edit_astcenc_config(
 			argidx++;
 			cli_config.silentmode = 1;
 		}
-		else
-			if (!strcmp(argv[argidx], "-v"))
-		{
-			argidx += 7;
-			if (argidx > argc)
-			{
-				printf("ERROR: -v switch with less than 6 arguments\n");
-				return 1;
-			}
-
-			config.v_rgba_radius = atoi(argv[argidx - 6]);
-			config.v_rgb_power = static_cast<float>(atof(argv[argidx - 5]));
-			config.v_rgb_base = static_cast<float>(atof(argv[argidx - 4]));
-			config.v_rgb_mean = static_cast<float>(atof(argv[argidx - 3]));
-			config.v_rgb_stdev = static_cast<float>(atof(argv[argidx - 2]));
-			config.v_rgba_mean_stdev_mix = static_cast<float>(atof(argv[argidx - 1]));
-		}
-		else if (!strcmp(argv[argidx], "-va"))
-		{
-			argidx += 5;
-			if (argidx > argc)
-			{
-				printf("ERROR: -va switch with less than 4 arguments\n");
-				return 1;
-			}
-
-			config.v_a_power= static_cast<float>(atof(argv[argidx - 4]));
-			config.v_a_base = static_cast<float>(atof(argv[argidx - 3]));
-			config.v_a_mean = static_cast<float>(atof(argv[argidx - 2]));
-			config.v_a_stdev = static_cast<float>(atof(argv[argidx - 1]));
-		}
 		else if (!strcmp(argv[argidx], "-cw"))
 		{
 			argidx += 5;
@@ -711,17 +682,6 @@ static int edit_astcenc_config(
 
 			config.a_scale_radius = atoi(argv[argidx - 1]);
 		}
-		else if (!strcmp(argv[argidx], "-b"))
-		{
-			argidx += 2;
-			if (argidx > argc)
-			{
-				printf("ERROR: -b switch with no argument\n");
-				return 1;
-			}
-
-			config.b_deblock_weight = static_cast<float>(atof(argv[argidx - 1]));
-		}
 		else if (!strcmp(argv[argidx], "-esw"))
 		{
 			argidx += 2;
@@ -771,6 +731,60 @@ static int edit_astcenc_config(
 			cli_config.swz_encode.b = swizzle_components[2];
 			cli_config.swz_encode.a = swizzle_components[3];
 		}
+		else if (!strcmp(argv[argidx], "-ssw"))
+		{
+			argidx += 2;
+			if (argidx > argc)
+			{
+				printf("ERROR: -ssw switch with no argument\n");
+				return 1;
+			}
+
+			size_t char_count = strlen(argv[argidx - 1]);
+			if (char_count == 0)
+			{
+				printf("ERROR: -ssw pattern contains no characters\n");
+				return 1;
+			}
+
+			if (char_count > 4)
+			{
+				printf("ERROR: -ssw pattern contains more than 4 characters\n");
+				return 1;
+			}
+
+			bool found_r = false;
+			bool found_g = false;
+			bool found_b = false;
+			bool found_a = false;
+
+			for (size_t i = 0; i < char_count; i++)
+			{
+				switch (argv[argidx - 1][i])
+				{
+				case 'r':
+					found_r = true;
+					break;
+				case 'g':
+					found_g = true;
+					break;
+				case 'b':
+					found_b = true;
+					break;
+				case 'a':
+					found_a = true;
+					break;
+				default:
+					printf("ERROR: -ssw component '%c' is not valid\n", argv[argidx - 1][i]);
+					return 1;
+				}
+			}
+
+			config.cw_r_weight = found_r ? 1.0f : 0.0f;
+			config.cw_g_weight = found_g ? 1.0f : 0.0f;
+			config.cw_b_weight = found_b ? 1.0f : 0.0f;
+			config.cw_a_weight = found_a ? 1.0f : 0.0f;
+		}
 		else if (!strcmp(argv[argidx], "-dsw"))
 		{
 			argidx += 2;
@@ -990,6 +1004,22 @@ static int edit_astcenc_config(
 
 			cli_config.thread_count = atoi(argv[argidx - 1]);
 		}
+		else if (!strcmp(argv[argidx], "-repeats"))
+		{
+			argidx += 2;
+			if (argidx > argc)
+			{
+				printf("ERROR: -repeats switch with no argument\n");
+				return 1;
+			}
+
+			cli_config.repeat_count = atoi(argv[argidx - 1]);
+			if (cli_config.repeat_count <= 0)
+			{
+				printf("ERROR: -repeats value must be at least one\n");
+				return 1;
+			}
+		}
 		else if (!strcmp(argv[argidx], "-yflip"))
 		{
 			argidx++;
@@ -1012,20 +1042,20 @@ static int edit_astcenc_config(
 				return 1;
 			}
 		}
-		// Option: Encode a 3D image from an array of 2D images.
-		else if (!strcmp(argv[argidx], "-array"))
+		// Option: Encode a 3D image from a sequence of 2D images.
+		else if (!strcmp(argv[argidx], "-zdim"))
 		{
 			// Only supports compressing
 			if (!(operation & ASTCENC_STAGE_COMPRESS))
 			{
-				printf("ERROR: -array switch is only valid for compression\n");
+				printf("ERROR: -zdim switch is only valid for compression\n");
 				return 1;
 			}
 
 			// Image depth must be specified.
 			if (argidx + 2 > argc)
 			{
-				printf("ERROR: -array switch with no argument\n");
+				printf("ERROR: -zdim switch with no argument\n");
 				return 1;
 			}
 			argidx++;
@@ -1033,30 +1063,35 @@ static int edit_astcenc_config(
 			// Read array size (image depth).
 			if (!sscanf(argv[argidx], "%u", &cli_config.array_size) || cli_config.array_size == 0)
 			{
-				printf("ERROR: -array size '%s' is invalid\n", argv[argidx]);
+				printf("ERROR: -zdim size '%s' is invalid\n", argv[argidx]);
 				return 1;
 			}
 
 			if ((cli_config.array_size > 1) && (config.block_z == 1))
 			{
-				printf("ERROR: -array with 3D input data for a 2D output format\n");
+				printf("ERROR: -zdim with 3D input data for a 2D output format\n");
 				return 1;
 			}
 			argidx++;
 		}
 #if defined(ASTCENC_DIAGNOSTICS)
-		else if (!strcmp(argv[argidx], "-dtrace-out"))
+		else if (!strcmp(argv[argidx], "-dtrace"))
 		{
 			argidx += 2;
 			if (argidx > argc)
 			{
-				printf("ERROR: -dtrace-out switch with no argument\n");
+				printf("ERROR: -dtrace switch with no argument\n");
 				return 1;
 			}
 
 			config.trace_file_path = argv[argidx - 1];
 		}
 #endif
+		else if (!strcmp(argv[argidx], "-dimage"))
+		{
+			argidx += 1;
+			cli_config.diagnostic_images = true;
+		}
 		else // check others as well
 		{
 			printf("ERROR: Argument '%s' not recognized\n", argv[argidx]);
@@ -1075,7 +1110,7 @@ static int edit_astcenc_config(
 
 	if (!config.trace_file_path)
 	{
-		printf("ERROR: Diagnostics builds must set -dtrace-out\n");
+		printf("ERROR: Diagnostics builds must set -dtrace\n");
 		return 1;
 	}
 #endif
@@ -1099,7 +1134,7 @@ static void print_astcenc_config(
 		printf("Compressor settings\n");
 		printf("===================\n\n");
 
-		switch(config.profile)
+		switch (config.profile)
 		{
 		case ASTCENC_PRF_LDR:
 			printf("    Color profile:              LDR linear\n");
@@ -1125,36 +1160,25 @@ static void print_astcenc_config(
 		}
 
 		printf("    Bitrate:                    %3.2f bpp\n", 128.0 / (config.block_x * config.block_y * config.block_z));
-
-		printf("    Radius mean/stdev:          %u texels\n", config.v_rgba_radius);
-		printf("    RGB power:                  %g\n", (double)config.v_rgb_power );
-		printf("    RGB base weight:            %g\n", (double)config.v_rgb_base);
-		printf("    RGB mean weight:            %g\n", (double)config.v_rgb_mean);
-		printf("    RGB stdev weight:           %g\n", (double)config.v_rgb_stdev);
-		printf("    RGB mean/stdev mixing:      %g\n", (double)config.v_rgba_mean_stdev_mix);
-		printf("    Alpha power:                %g\n", (double)config.v_a_power);
-		printf("    Alpha base weight:          %g\n", (double)config.v_a_base);
-		printf("    Alpha mean weight:          %g\n", (double)config.v_a_mean);
-		printf("    Alpha stdev weight:         %g\n", (double)config.v_a_stdev);
 		printf("    RGB alpha scale weight:     %d\n", (config.flags & ASTCENC_FLG_USE_ALPHA_WEIGHT));
 		if ((config.flags & ASTCENC_FLG_USE_ALPHA_WEIGHT))
 		{
 			printf("    Radius RGB alpha scale:     %u texels\n", config.a_scale_radius);
 		}
 
-		printf("    R component weight:         %g\n",(double)config.cw_r_weight);
-		printf("    G component weight:         %g\n",(double)config.cw_g_weight);
-		printf("    B component weight:         %g\n",(double)config.cw_b_weight);
-		printf("    A component weight:         %g\n",(double)config.cw_a_weight);
-		printf("    Deblock artifact setting:   %g\n", (double)config.b_deblock_weight);
+		printf("    R component weight:         %g\n", static_cast<double>(config.cw_r_weight));
+		printf("    G component weight:         %g\n", static_cast<double>(config.cw_g_weight));
+		printf("    B component weight:         %g\n", static_cast<double>(config.cw_b_weight));
+		printf("    A component weight:         %g\n", static_cast<double>(config.cw_a_weight));
 		printf("    Partition cutoff:           %u partitions\n", config.tune_partition_count_limit);
 		printf("    Partition index cutoff:     %u partition ids\n", config.tune_partition_index_limit);
-		printf("    PSNR cutoff:                %g dB\n", (double)config.tune_db_limit);
-		printf("    2.2+ partition cutoff:      %g\n", (double)config.tune_2_partition_early_out_limit_factor);
-		printf("    3.2+ partition cutoff:      %g\n", (double)config.tune_3_partition_early_out_limit_factor);
-		printf("    2 plane correlation cutoff: %g\n", (double)config.tune_2_plane_early_out_limit_correlation);
-		printf("    Block mode centile cutoff:  %g%%\n", (double)(config.tune_block_mode_limit));
-		printf("    Max refinement cutoff:      %u iterations\n", config.tune_refinement_limit);
+		printf("    PSNR cutoff:                %g dB\n", static_cast<double>(config.tune_db_limit));
+		printf("    3 partition cutoff:         %g\n", static_cast<double>(config.tune_2_partition_early_out_limit_factor));
+		printf("    4 partition cutoff:         %g\n", static_cast<double>(config.tune_3_partition_early_out_limit_factor));
+		printf("    2 plane correlation cutoff: %g\n", static_cast<double>(config.tune_2_plane_early_out_limit_correlation));
+		printf("    Block mode centile cutoff:  %g%%\n", static_cast<double>(config.tune_block_mode_limit));
+		printf("    Candidate cutoff:           %u candidates\n", config.tune_candidate_limit);
+		printf("    Refinement cutoff:          %u iterations\n", config.tune_refinement_limit);
 		printf("    Compressor thread count:    %d\n", cli_config.thread_count);
 		printf("\n");
 	}
@@ -1253,6 +1277,30 @@ static void image_set_pixel(
 	data[(4 * img.dim_x * y) + (4 * x + 3)] = pixel.lane<3>();
 }
 
+/**
+ * @brief Set the value of a single pixel in an image.
+ *
+ * @param[out] img     The output image; must use F32 texture components.
+ * @param      x       The pixel x coordinate.
+ * @param      y       The pixel y coordinate.
+ * @param      pixel   The pixel color value to write.
+ */
+static void image_set_pixel_u8(
+	astcenc_image& img,
+	size_t x,
+	size_t y,
+	vint4 pixel
+) {
+	// We should never escape bounds
+	assert(x < img.dim_x);
+	assert(y < img.dim_y);
+	assert(img.data_type == ASTCENC_TYPE_U8);
+
+	uint8_t* data = static_cast<uint8_t*>(img.data[0]);
+	pixel = pack_low_bytes(pixel);
+	store_nbytes(pixel, data + (4 * img.dim_x * y) + (4 * x    ));
+}
+
 /**
  * @brief Create a copy of @c input with forced unit-length normal vectors.
  *
@@ -1382,6 +1430,365 @@ static void image_preprocess_premultiply(
 	}
 }
 
+/**
+ * @brief Populate a single diagnostic image showing aspects of the encoding.
+ *
+ * @param context      The context to use.
+ * @param image        The compressed image to analyze.
+ * @param diag_image   The output visualization image to populate.
+ * @param texel_func   The per-texel callback used to determine output color.
+ */
+static void print_diagnostic_image(
+	astcenc_context* context,
+	const astc_compressed_image& image,
+	astcenc_image& diag_image,
+	std::function<vint4(astcenc_block_info&, size_t, size_t)> texel_func
+) {
+	size_t block_cols = (image.dim_x + image.block_x - 1) / image.block_x;
+	size_t block_rows = (image.dim_y + image.block_y - 1) / image.block_y;
+
+	uint8_t* data = image.data;
+	for (size_t block_y = 0; block_y < block_rows; block_y++)
+	{
+		for (size_t block_x = 0; block_x < block_cols; block_x++)
+		{
+			astcenc_block_info block_info;
+			astcenc_get_block_info(context, data, &block_info);
+			data += 16;
+
+			size_t start_row = block_y * image.block_y;
+			size_t start_col = block_x * image.block_x;
+
+			size_t end_row = astc::min(start_row + image.block_y, static_cast<size_t>(image.dim_y));
+			size_t end_col = astc::min(start_col + image.block_x, static_cast<size_t>(image.dim_x));
+
+			for (size_t texel_y = start_row; texel_y < end_row; texel_y++)
+			{
+				for (size_t texel_x = start_col; texel_x < end_col; texel_x++)
+				{
+					vint4 color = texel_func(block_info, texel_x - start_col, texel_y - start_row);
+					image_set_pixel_u8(diag_image, texel_x, texel_y, color);
+				}
+			}
+		}
+	}
+}
+
+/**
+ * @brief Print a set of diagnostic images showing aspects of the encoding.
+ *
+ * @param context       The context to use.
+ * @param image         The compressed image to analyze.
+ * @param output_file   The output file name to use as a stem for new names.
+ */
+static void print_diagnostic_images(
+	astcenc_context* context,
+	const astc_compressed_image& image,
+	const std::string& output_file
+) {
+	if (image.dim_z != 1)
+	{
+		return;
+	}
+
+	// Try to find a file extension we know about
+	size_t index = output_file.find_last_of(".");
+	std::string stem = output_file;
+	if (index != std::string::npos)
+	{
+		stem = stem.substr(0, index);
+	}
+
+	auto diag_image = alloc_image(8, image.dim_x, image.dim_y, image.dim_z);
+
+	// ---- ---- ---- ---- Partitioning ---- ---- ---- ----
+	auto partition_func = [](astcenc_block_info& info, size_t texel_x, size_t texel_y) {
+		const vint4 colors[] {
+			vint4(  0,   0,   0, 255),
+			vint4(255,   0,   0, 255),
+			vint4(  0, 255,   0, 255),
+			vint4(  0,   0, 255, 255),
+			vint4(255, 255, 255, 255)
+		};
+
+		size_t texel_index = texel_y * info.block_x + texel_x;
+
+		int partition { 0 };
+		if (!info.is_constant_block)
+		{
+			partition = info.partition_assignment[texel_index] + 1;
+		}
+
+		return colors[partition];
+	};
+
+	print_diagnostic_image(context, image, *diag_image, partition_func);
+	std::string fname = stem + "_diag_partitioning.png";
+	store_ncimage(diag_image, fname.c_str(), false);
+
+	// ---- ---- ---- ---- Weight planes  ---- ---- ---- ----
+	auto texel_func1 = [](astcenc_block_info& info, size_t texel_x, size_t texel_y) {
+		(void)texel_x;
+		(void)texel_y;
+
+		const vint4 colors[] {
+			vint4(  0,   0,   0, 255),
+			vint4(255,   0,   0, 255),
+			vint4(  0, 255,   0, 255),
+			vint4(  0,   0, 255, 255),
+			vint4(255, 255, 255, 255)
+		};
+
+		int component { 0 };
+		if (info.is_dual_plane_block)
+		{
+			component = info.dual_plane_component + 1;
+		}
+
+		return colors[component];
+	};
+
+	print_diagnostic_image(context, image, *diag_image, texel_func1);
+	fname = stem + "_diag_weight_plane2.png";
+	store_ncimage(diag_image, fname.c_str(), false);
+
+	// ---- ---- ---- ---- Weight density  ---- ---- ---- ----
+	auto texel_func2 = [](astcenc_block_info& info, size_t texel_x, size_t texel_y) {
+		(void)texel_x;
+		(void)texel_y;
+
+		float density = 0.0f;
+		if (!info.is_constant_block)
+		{
+			float texel_count = static_cast<float>(info.block_x * info.block_y);
+			float weight_count = static_cast<float>(info.weight_x * info.weight_y);
+			density = weight_count / texel_count;
+		}
+
+		int densityi = static_cast<int>(255.0f * density);
+		return vint4(densityi, densityi, densityi, 255);
+	};
+
+	print_diagnostic_image(context, image, *diag_image, texel_func2);
+	fname = stem + "_diag_weight_density.png";
+	store_ncimage(diag_image, fname.c_str(), false);
+
+	// ---- ---- ---- ---- Weight quant  ---- ---- ---- ----
+	auto texel_func3 = [](astcenc_block_info& info, size_t texel_x, size_t texel_y) {
+		(void)texel_x;
+		(void)texel_y;
+
+		int quant { 0 };
+		if (!info.is_constant_block)
+		{
+			quant = info.weight_level_count - 1;
+		}
+
+		return vint4(quant, quant, quant, 255);
+	};
+
+	print_diagnostic_image(context, image, *diag_image, texel_func3);
+	fname = stem + "_diag_weight_quant.png";
+	store_ncimage(diag_image, fname.c_str(), false);
+
+	// ---- ---- ---- ---- Color quant  ---- ---- ---- ----
+	auto texel_func4 = [](astcenc_block_info& info, size_t texel_x, size_t texel_y) {
+		(void)texel_x;
+		(void)texel_y;
+
+		int quant { 0 };
+		if (!info.is_constant_block)
+		{
+			quant = info.color_level_count - 1;
+		}
+
+		return vint4(quant, quant, quant, 255);
+	};
+
+	print_diagnostic_image(context, image, *diag_image, texel_func4);
+	fname = stem + "_diag_color_quant.png";
+	store_ncimage(diag_image, fname.c_str(), false);
+
+	// ---- ---- ---- ---- Color endpoint mode: Index ---- ---- ---- ----
+	auto texel_func5 = [](astcenc_block_info& info, size_t texel_x, size_t texel_y) {
+		(void)texel_x;
+		(void)texel_y;
+
+		size_t texel_index = texel_y * info.block_x + texel_x;
+
+		int cem { 255 };
+		if (!info.is_constant_block)
+		{
+			uint8_t partition = info.partition_assignment[texel_index];
+			cem = info.color_endpoint_modes[partition] * 16;
+		}
+
+		return vint4(cem, cem, cem, 255);
+	};
+
+	print_diagnostic_image(context, image, *diag_image, texel_func5);
+	fname = stem + "_diag_cem_index.png";
+	store_ncimage(diag_image, fname.c_str(), false);
+
+	// ---- ---- ---- ---- Color endpoint mode: Components ---- ---- ---- ----
+	auto texel_func6 = [](astcenc_block_info& info, size_t texel_x, size_t texel_y) {
+		(void)texel_x;
+		(void)texel_y;
+
+		const vint4 colors[] {
+			vint4(  0,   0,   0, 255),
+			vint4(255,   0,   0, 255),
+			vint4(  0, 255,   0, 255),
+			vint4(  0,   0, 255, 255),
+			vint4(255, 255, 255, 255)
+		};
+
+		size_t texel_index = texel_y * info.block_x + texel_x;
+
+		int components { 0 };
+		if (!info.is_constant_block)
+		{
+			uint8_t partition = info.partition_assignment[texel_index];
+			uint8_t cem = info.color_endpoint_modes[partition];
+
+			switch (cem)
+			{
+				case 0:
+				case 1:
+				case 2:
+				case 3:
+					components = 1;
+					break;
+				case 4:
+				case 5:
+					components = 2;
+					break;
+				case 6:
+				case 7:
+				case 8:
+				case 9:
+				case 11:
+					components = 3;
+					break;
+				default:
+					components = 4;
+					break;
+			}
+		}
+
+		return colors[components];
+	};
+
+	print_diagnostic_image(context, image, *diag_image, texel_func6);
+	fname = stem + "_diag_cem_components.png";
+	store_ncimage(diag_image, fname.c_str(), false);
+
+	// ---- ---- ---- ---- Color endpoint mode: Style ---- ---- ---- ----
+	auto texel_func7 = [](astcenc_block_info& info, size_t texel_x, size_t texel_y) {
+		(void)texel_x;
+		(void)texel_y;
+
+		const vint4 colors[] {
+			vint4(  0,   0,   0, 255),
+			vint4(255,   0,   0, 255),
+			vint4(  0, 255,   0, 255),
+			vint4(  0,   0, 255, 255),
+		};
+
+		size_t texel_index = texel_y * info.block_x + texel_x;
+
+		int style { 0 };
+		if (!info.is_constant_block)
+		{
+			uint8_t partition = info.partition_assignment[texel_index];
+			uint8_t cem = info.color_endpoint_modes[partition];
+
+			switch (cem)
+			{
+				// Direct - two absolute endpoints
+				case 0:
+				case 1:
+				case 2:
+				case 3:
+				case 4:
+				case 8:
+				case 11:
+				case 12:
+				case 14:
+				case 15:
+					style = 1;
+					break;
+				// Offset - one absolute plus delta
+				case 5:
+				case 9:
+				case 13:
+					style = 2;
+					break;
+				// Scale - one absolute plus scale
+				case 6:
+				case 7:
+				case 10:
+					style = 3;
+					break;
+				// Shouldn't happen ...
+				default:
+					style = 0;
+					break;
+			}
+		}
+
+		return colors[style];
+	};
+
+	print_diagnostic_image(context, image, *diag_image, texel_func7);
+	fname = stem + "_diag_cem_style.png";
+	store_ncimage(diag_image, fname.c_str(), false);
+
+	// ---- ---- ---- ---- Color endpoint mode: Style ---- ---- ---- ----
+	auto texel_func8 = [](astcenc_block_info& info, size_t texel_x, size_t texel_y) {
+		(void)texel_x;
+		(void)texel_y;
+
+		size_t texel_index = texel_y * info.block_x + texel_x;
+
+		int style { 0 };
+		if (!info.is_constant_block)
+		{
+			uint8_t partition = info.partition_assignment[texel_index];
+			uint8_t cem = info.color_endpoint_modes[partition];
+
+			switch (cem)
+			{
+				// LDR blocks
+				case 0:
+				case 1:
+				case 4:
+				case 5:
+				case 6:
+				case 8:
+				case 9:
+				case 10:
+				case 12:
+				case 13:
+					style = 128;
+					break;
+				// HDR blocks
+				default:
+					style = 155;
+					break;
+			}
+		}
+
+		return vint4(style, style, style, 255);
+	};
+
+	print_diagnostic_image(context, image, *diag_image, texel_func8);
+	fname = stem + "_diag_cem_hdr.png";
+	store_ncimage(diag_image, fname.c_str(), false);
+
+	free_image(diag_image);
+}
+
 /**
  * @brief The main entry point.
  *
@@ -1422,7 +1829,6 @@ int main(
 		break;
 	}
 
-
 	std::string input_filename = argc >= 3 ? argv[2] : "";
 	std::string output_filename = argc >= 4 ? argv[3] : "";
 
@@ -1488,7 +1894,7 @@ int main(
 	}
 
 	// Initialize cli_config_options with default values
-	cli_config_options cli_config { 0, 1, false, false, -10, 10,
+	cli_config_options cli_config { 0, 1, 1, false, false, false, -10, 10,
 		{ ASTCENC_SWZ_R, ASTCENC_SWZ_G, ASTCENC_SWZ_B, ASTCENC_SWZ_A },
 		{ ASTCENC_SWZ_R, ASTCENC_SWZ_G, ASTCENC_SWZ_B, ASTCENC_SWZ_A } };
 
@@ -1512,8 +1918,11 @@ int main(
 	if (operation & ASTCENC_STAGE_ST_NCOMP)
 	{
 		int bitness = get_output_filename_enforced_bitness(output_filename.c_str());
-		if (bitness == -1)
+		if (bitness < 0)
 		{
+			const char *eptr = strrchr(output_filename.c_str(), '.');
+			eptr = eptr ? eptr : "";
+			printf("ERROR: Unknown uncompressed output file type '%s'\n", eptr);
 			return 1;
 		}
 	}
@@ -1528,7 +1937,9 @@ int main(
 
 		if (!(is_null || ends_with(output_filename, ".astc") || ends_with(output_filename, ".ktx")))
 		{
-			printf("ERROR: Unknown compressed output file type\n");
+			const char *eptr = strrchr(output_filename.c_str(), '.');
+			eptr = eptr ? eptr : "";
+			printf("ERROR: Unknown compressed output file type '%s'\n", eptr);
 			return 1;
 		}
 	}
@@ -1603,23 +2014,23 @@ int main(
 		}
 	}
 
-	double start_coding_time = get_time();
-
 	double image_size = 0.0;
 	if (image_uncomp_in)
 	{
-		image_size = (double)image_uncomp_in->dim_x *
-		             (double)image_uncomp_in->dim_y *
-		             (double)image_uncomp_in->dim_z;
+		image_size = static_cast<double>(image_uncomp_in->dim_x) *
+		             static_cast<double>(image_uncomp_in->dim_y) *
+		             static_cast<double>(image_uncomp_in->dim_z);
 	}
 	else
 	{
-		image_size = (double)image_comp.dim_x *
-		             (double)image_comp.dim_y *
-		             (double)image_comp.dim_z;
+		image_size = static_cast<double>(image_comp.dim_x) *
+		             static_cast<double>(image_comp.dim_y) *
+		             static_cast<double>(image_comp.dim_z);
 	}
 
 	// Compress an image
+	double best_compression_time = 100000.0;
+	double total_compression_time = 0.0;
 	if (operation & ASTCENC_STAGE_COMPRESS)
 	{
 		print_astcenc_config(cli_config, config);
@@ -1640,16 +2051,27 @@ int main(
 
 		// Only launch worker threads for multi-threaded use - it makes basic
 		// single-threaded profiling and debugging a little less convoluted
-		if (cli_config.thread_count > 1)
+		double start_compression_time = get_time();
+		for (unsigned int i = 0; i < cli_config.repeat_count; i++)
 		{
-			launch_threads(cli_config.thread_count, compression_workload_runner, &work);
-		}
-		else
-		{
-			work.error = astcenc_compress_image(
-			    work.context, work.image, &work.swizzle,
-			    work.data_out, work.data_len, 0);
+			double start_iter_time = get_time();
+			if (cli_config.thread_count > 1)
+			{
+				launch_threads(cli_config.thread_count, compression_workload_runner, &work);
+			}
+			else
+			{
+				work.error = astcenc_compress_image(
+					work.context, work.image, &work.swizzle,
+					work.data_out, work.data_len, 0);
+			}
+
+			astcenc_compress_reset(codec_context);
+
+			double iter_time = get_time() - start_iter_time;
+			best_compression_time = astc::min(iter_time, best_compression_time);
 		}
+		total_compression_time = get_time() - start_compression_time;
 
 		if (work.error != ASTCENC_SUCCESS)
 		{
@@ -1668,6 +2090,8 @@ int main(
 	}
 
 	// Decompress an image
+	double best_decompression_time = 100000.0;
+	double total_decompression_time = 0.0;
 	if (operation & ASTCENC_STAGE_DECOMPRESS)
 	{
 		int out_bitness = get_output_filename_enforced_bitness(output_filename.c_str());
@@ -1690,16 +2114,27 @@ int main(
 
 		// Only launch worker threads for multi-threaded use - it makes basic
 		// single-threaded profiling and debugging a little less convoluted
-		if (cli_config.thread_count > 1)
+		double start_decompression_time = get_time();
+		for (unsigned int i = 0; i < cli_config.repeat_count; i++)
 		{
-			launch_threads(cli_config.thread_count, decompression_workload_runner, &work);
-		}
-		else
-		{
-			work.error = astcenc_decompress_image(
-			    work.context, work.data, work.data_len,
-			    work.image_out, &work.swizzle, 0);
+			double start_iter_time = get_time();
+			if (cli_config.thread_count > 1)
+			{
+				launch_threads(cli_config.thread_count, decompression_workload_runner, &work);
+			}
+			else
+			{
+				work.error = astcenc_decompress_image(
+				    work.context, work.data, work.data_len,
+				    work.image_out, &work.swizzle, 0);
+			}
+
+			astcenc_decompress_reset(codec_context);
+
+			double iter_time = get_time() - start_iter_time;
+			best_decompression_time = astc::min(iter_time, best_decompression_time);
 		}
+		total_decompression_time = get_time() - start_decompression_time;
 
 		if (work.error != ASTCENC_SUCCESS)
 		{
@@ -1708,9 +2143,13 @@ int main(
 		}
 	}
 
-	double end_coding_time = get_time();
+#if defined(_WIN32)
+	bool is_null = output_filename == "NUL" || output_filename == "nul";
+#else
+	bool is_null = output_filename == "/dev/null";
+#endif
 
-	// Print metrics in comparison mode
+   // Print metrics in comparison mode
 	if (operation & ASTCENC_STAGE_COMPARE)
 	{
 		bool is_normal_map = config.flags & ASTCENC_FLG_MAP_NORMAL;
@@ -1744,11 +2183,6 @@ int main(
 		}
 		else
 		{
-#if defined(_WIN32)
-			bool is_null = output_filename == "NUL" || output_filename == "nul";
-#else
-			bool is_null = output_filename == "/dev/null";
-#endif
 			if (!is_null)
 			{
 				printf("ERROR: Unknown compressed output file type\n");
@@ -1760,12 +2194,6 @@ int main(
 	// Store decompressed image
 	if (operation & ASTCENC_STAGE_ST_NCOMP)
 	{
-#if defined(_WIN32)
-		bool is_null = output_filename == "NUL" || output_filename == "nul";
-#else
-		bool is_null = output_filename == "/dev/null";
-#endif
-
 		if (!is_null)
 		{
 			bool store_result = store_ncimage(image_decomp_out, output_filename.c_str(),
@@ -1778,6 +2206,12 @@ int main(
 		}
 	}
 
+	// Store diagnostic images
+	if (cli_config.diagnostic_images && !is_null)
+	{
+		print_diagnostic_images(codec_context, image_comp, output_filename);
+	}
+
 	free_image(image_uncomp_in);
 	free_image(image_decomp_out);
 	astcenc_context_free(codec_context);
@@ -1787,14 +2221,30 @@ int main(
 	if ((operation & ASTCENC_STAGE_COMPARE) || (!cli_config.silentmode))
 	{
 		double end_time = get_time();
-		double tex_rate = image_size / (end_coding_time - start_coding_time);
-		tex_rate = tex_rate / 1000000.0;
+
+		double repeats = static_cast<double>(cli_config.repeat_count);
+		double avg_compression_time = total_compression_time / repeats;
+		double avg_decompression_time = total_decompression_time / repeats;
+		double total_time = (end_time - start_time) - ((repeats - 1.0) * avg_compression_time)  - ((repeats - 1.0) * avg_decompression_time);
 
 		printf("Performance metrics\n");
 		printf("===================\n\n");
-		printf("    Total time:                %8.4f s\n", end_time - start_time);
-		printf("    Coding time:               %8.4f s\n", end_coding_time - start_coding_time);
-		printf("    Coding rate:               %8.4f MT/s\n", tex_rate);
+		printf("    Total time:                %8.4f s\n", total_time);
+
+		if (operation & ASTCENC_STAGE_COMPRESS)
+		{
+			double compression_rate = image_size / (best_compression_time * 1000000.0);
+
+			printf("    Coding time:               %8.4f s\n", best_compression_time);
+			printf("    Coding rate:               %8.4f MT/s\n", compression_rate);
+		}
+
+		if (operation & ASTCENC_STAGE_DECOMPRESS)
+		{
+			double decompression_rate = image_size / (best_decompression_time * 1000000.0);
+			printf("    Decoding time:             %8.4f s\n", best_decompression_time);
+			printf("    Decoding rate:             %8.4f MT/s\n", decompression_rate);
+		}
 	}
 
 	return 0;
