@@ -1,6 +1,6 @@
 // SPDX-License-Identifier: Apache-2.0
 // ----------------------------------------------------------------------------
-// Copyright 2011-2021 Arm Limited
+// Copyright 2011-2022 Arm Limited
 //
 // Licensed under the Apache License, Version 2.0 (the "License"); you may not
 // use this file except in compliance with the License. You may obtain a copy
@@ -25,7 +25,7 @@
 /** @brief The version header. */
 static const char *astcenc_copyright_string =
 R"(astcenc v%s, %u-bit %s%s%s
-Copyright 2011-%s Arm Limited, all rights reserved
+Copyright (c) 2011-%s Arm Limited. All rights reserved.
 )";
 
 /** @brief The short-form help text. */
@@ -161,14 +161,13 @@ COMPRESSION
        -mask
            The input texture is a mask texture with unrelated data stored
            in the various color components, so enable error heuristics that
-           aim to improve perceptual quality by minimizing the effect of
-           error cross-talk across the color components.
+           aim to improve quality by minimizing the effect of error
+           cross-talk across the color components.
 
        -normal
            The input texture is a three component linear LDR normal map
            storing unit length normals as (R=X, G=Y, B=Z). The output will
-           be a two component X+Y normal map stored as (RGB=X, A=Y),
-           optimized for angular error instead of simple PSNR. The Z
+           be a two component X+Y normal map stored as (RGB=X, A=Y). The Z
            component can be recovered programmatically in shader code by
            using the equation:
 
@@ -176,6 +175,9 @@ COMPRESSION
                nml.xy = nml.xy * 2.0 - 1.0;           // Unpack to [-1,1]
                nml.z = sqrt(1 - dot(nml.xy, nml.xy)); // Compute Z
 
+           Alternative component swizzles can be set with -esw and -dsw
+           parameters.
+
        -rgbm <max>
            The input texture is an RGBM encoded texture, storing values HDR
            values between 0 and <max> in an LDR container format with a
@@ -194,8 +196,8 @@ COMPRESSION
            typically lowers the measured PSNR score. Perceptual methods are
            currently only available for normal maps and RGB color data.
 
-       -array <size>
-           Loads an array of <size> 2D image slices to use as a 3D image.
+       -zdim <zdim>
+           Load a sequence of <zdim> 2D image slices to use as a 3D image.
            The input filename given is used is decorated with the postfix
            "_<slice>" to find the file to load. For example, an input named
            "input.png" would load as input_0.png, input_1.png, etc.
@@ -218,17 +220,8 @@ R"(
 COMPRESSION TIPS & TRICKS
        ASTC is a block-based format that can be prone to block artifacts.
        If block artifacts are a problem when compressing a given texture,
-       adding some or all of following command-line options may help:
-
-           -b 1.8
-           -v 2 1 1 0 25 0.1
-           -va 1 1 0 25
-           -dblimit 60
-
-       The -b option is a general-purpose block-artifact reduction option.
-       The -v and -va option settings will concentrate effort where smooth
-       regions lie next to regions with high detail, which are particularly
-       prone to block artifacts.
+       increasing the compressor quality preset can help to alleviate the
+       problem.
 
        If a texture exhibits severe block artifacts in only some of the
        color components, which is a common problem for mask textures, then
@@ -243,34 +236,6 @@ ADVANCED COMPRESSION
        These options provide low-level control of the codec error metric
        computation, used to determine what good compression looks like.
 
-       -v <radius> <power> <base> <mean> <stdev> <mix>
-           Compute the per-texel relative error weighting for the RGB color
-           components as follows:
-
-           weight = 1 / (<base> + <mean> * mean^2 + <stdev> * stdev^2)
-
-           The <radius> argument specifies the texel radius of the
-           neighborhood over which the average and standard deviation are
-           computed.
-
-           The <mix> parameter is used to control the degree of mixing of
-           the average and stddev error values across the color components.
-           Setting this parameter to 0 causes the computation to be done
-           separately for each color component; setting it to 1 causes the
-           results from the RGB components to be combined and applied to
-           all three components. Intermediate values between these two
-           settings do a linear mix of the two.
-
-           The <power> argument is a power used to raise the values of the
-           input texels before computing average and standard deviation;
-           e.g. a power of 0.5 causes the codec to take the square root
-           of every input texel value.
-
-       -va <power> <base> <mean> <stdev>
-           Compute the per-texel relative error weighting for the alpha
-           component, when used in conjunction with -v. See documentation
-           of -v for individual parameter documentation.
-
        -a <radius>
            For textures with alpha component, scale per-texel weights by
            the alpha value. The alpha value chosen for scaling of any
@@ -290,12 +255,6 @@ ADVANCED COMPRESSION
            significance, and values below 1 to decrease it. Set to 0 to
            exclude a component from error computation.
 
-       -b <weight>
-           Assign an additional weight scaling for texels at compression
-           block edges and corners. Setting this to a value above 1
-           increases the significance of texels closer to the edges of a
-           block, and can help to reduce block artifacts.
-
        -mpsnr <low> <high>
            Set the low and high f-stop values for the mPSNR error metric.
            The mPSNR error metric only applies to HDR textures.
@@ -426,33 +385,43 @@ ADVANCED COMPRESSION
        -------------
 
        -esw <swizzle>
-           Swizzle the color components before compression. The swizzle is
-           specified using a 4-character string, which defines the output
-           format ordering. The characters may be taken from the set
-           [rgba01], selecting either input color components or a literal
-           zero or one. For example to swap the RG components, and replace
-           alpha with 1, the swizzle 'grb1' should be used.
+           Specify an encoding swizzle to reorder the color components
+           before compression. The swizzle is specified using a four
+           character string, which defines the format ordering used by
+           the compressor.
 
-           The input swizzle takes place before any compression, and all
-           error weighting applied using the -cw option is applied to the
-           post-swizzle component ordering.
+           The characters may be taken from the set [rgba01], selecting
+           either input color components or a literal zero or one. For
+           example to swap the RG components, and replace alpha with 1,
+           the swizzle 'grb1' should be used.
 
            By default all 4 post-swizzle components are included in the
-           error metrics during compression. When using -esw to map two
+           compression error metrics. When using -esw to map two
            component data to the L+A endpoint (e.g. -esw rrrg) the
            luminance data stored in the RGB components will be weighted 3
            times more strongly than the alpha component. This can be
-           corrected using the -cw option to zero the weights of unused
-           components; e.g. using -cw 1 0 0 1.
+           corrected using the -ssw option to specify which components
+           will be sampled at runtime e.g. -ssw ra.
+
+       -ssw <swizzle>
+           Specify a sampling swizzle to identify which color components
+           are actually read by the application shader program. For example,
+           using -ssw ra tells the compressor that the green and blue error
+           does not matter because the data is not actually read.
+
+           The sampling swizzle is based on the channel ordering after the
+           -esw transform has been applied. Note -ssw exposes the same
+           functionality as -cw, but in a more user-friendly form.
 
        -dsw <swizzle>
-           Swizzle the color components after decompression. The swizzle is
-           specified using the same method as the -esw option, with support
-           for an additional "z" character. This is used to specify that
-           the compressed data stores an X+Y normal map, and that the Z
-           output component should be reconstructed from the two components
-           stored in the data. For the typical ASTC normal encoding, which
-           uses an 'rrrg' compression swizzle, you should specify an 'raz1'
+           Specify a decompression swizzle used to reorder the color
+           components after decompression. The swizzle is specified using
+           the same method as the -esw option, with support for an extra
+           "z" character. This is used to specify that the compressed data
+           stores an X+Y normal map, and that the Z output component
+           should be reconstructed from the two components stored in the
+           data. For the typical ASTC normal encoding, which uses an
+           'rrrg' compression swizzle, you should specify an 'raz1'
            swizzle for decompression.
 
        -yflip
@@ -601,7 +570,7 @@ void astcenc_print_header()
 	const char* f16ctype = "";
 #endif
 
-	unsigned int bits = (int)(sizeof(void*) * 8);
+	unsigned int bits = static_cast<unsigned int>(sizeof(void*) * 8);
 	printf(astcenc_copyright_string,
 	       VERSION_STRING, bits, simdtype, pcnttype, f16ctype, YEAR_STRING);
 }
